import re
from datetime import datetime

def extract_name_from_patient_line(text):
    """Extracts first and last name from typical patient lines in various formats."""
    patterns = [
        # Format: Patient: Last name, First name geb. DD.MM.YYYY Fallnummer: XXXXXXXX
        r"(?:Patient:|Patientin:|Pat\.:|Pat:)[\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]*[,][\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]*(?:geb\.|geboren am:)[\s]*(\d{1,2}\.\d{1,2}\.\d{4})(?:.*?(?:Fallnummer:|Fallnr\.:)[\s]*(\d+))?",
        
        # Format: Patient: Last name First name geb. DD.MM.YYYY Fallnummer: XXXXXXXX
        r"(?:Patient:|Patientin:|Pat\.:|Pat:)[\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]+([a-zA-ZäöüÄÖÜß\-]+)[\s]*(?:geb\.|geboren am:)[\s]*(\d{1,2}\.\d{1,2}\.\d{4})(?:.*?(?:Fallnummer:|Fallnr\.:)[\s]*(\d+))?",
        
        # Format: Patient: Last name, First name
        r"(?:Patient:|Patientin:|Pat\.:|Pat:)[\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]*[,][\s]*([a-zA-ZäöüÄÖÜß\-]+)",
        
        # Format: Patient: Last name First name
        r"(?:Patient:|Patientin:|Pat\.:|Pat:)[\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]+([a-zA-ZäöüÄÖÜß\-]+)",

        # Format: Pat.: Last name, Dr. First name Geb.Dat.: DD.MM.YYYY
        r"(?:Pat\.:|Patient:)[\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]*,[\s]*(Dr\.)[\s]*([a-zA-ZäöüÄÖÜß\-]+)[\s]*Geb\.Dat\.:?[\s]*(\d{1,2}\.\d{1,2}\.\d{4})"
    ]
    
    for pattern in patterns:
        match = re.search(pattern, text)
        if match:
            groups = match.groups()
            last_name = groups[0].strip() if groups[0] else "UNBEKANNT"
            title = groups[1].strip() if len(groups) > 1 and groups[1] else ""
            first_name = groups[2].strip() if len(groups) > 2 and groups[2] else "UNBEKANNT"
            birthdate = None
            if len(groups) >= 4 and groups[3]:
                try:
                    birthdate = datetime.strptime(groups[3].strip(), '%d.%m.%Y').strftime('%Y-%m-%d')
                except (ValueError, AttributeError):
                    pass
            return first_name, last_name, birthdate, title
    
    # Standardwerte, wenn keine Übereinstimmung gefunden wird
    return "UNBEKANNT", "UNBEKANNT", None, None

def validate_patient_info(info_dict):
    """Überprüft und korrigiert Patientendaten.
    
    Args:
        info_dict (dict): Dictionary mit Patienteninformationen
        
    Returns:
        dict: Validiertes und korrigiertes Dictionary
    """
    if not isinstance(info_dict, dict):
        return {
            'first_name': "UNBEKANNT",
            'last_name': "UNBEKANNT",
            'birthdate': None,
            'casenumber': None,
            'gender': "Unbekannt"
        }
    
    # Stelle sicher, dass erforderliche Schlüssel existieren
    for key in ['first_name', 'last_name']:
        if key not in info_dict or not info_dict[key] or info_dict[key] == "NOT FOUND":
            info_dict[key] = "UNBEKANNT"
    
    # Normalisiere Geschlecht
    if 'gender' in info_dict:
        if info_dict['gender'] == "male":
            info_dict['gender'] = "Männlich"
        elif info_dict['gender'] == "female":
            info_dict['gender'] = "Weiblich"
        elif info_dict['gender'] in ["unknown", "NOT FOUND"]:
            info_dict['gender'] = "Unbekannt"
    else:
        info_dict['gender'] = "Unbekannt"
        
    return info_dict



from django.db import models
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from ...administration.person.patient import Patient
    from ..laboratory import LabValue
    from ...other.unit import Unit
    from .patient_lab_sample import PatientLabSample
    from endoreg_db.utils.links.requirement_link import RequirementLinks # Added import

class PatientLabValue(models.Model):
    """
    A class representing a patient lab value.

    Attributes:
        patient (Patient): The patient.
        lab_value (LabValue): The lab value.
        value (float): The value of the lab value.
        date (datetime): The date of the lab value.
    """
    patient = models.ForeignKey('Patient', on_delete=models.CASCADE,
                                related_name="lab_values", blank=True, null=True
    )
    lab_value = models.ForeignKey('LabValue', on_delete=models.CASCADE)
    value = models.FloatField(blank=True, null=True)
    value_str = models.CharField(max_length=255, blank=True, null=True)
    sample: models.ForeignKey["PatientLabSample"] = models.ForeignKey(
        'PatientLabSample', on_delete=models.CASCADE, 
        blank=True, null=True,
        related_name='values'
    )
    datetime = models.DateTimeField(# if not set, use now
        auto_now_add=True        
    )
    normal_range = models.JSONField(
        default = dict
    )
    unit = models.ForeignKey('Unit', on_delete=models.CASCADE, blank=True, null=True)

    if TYPE_CHECKING:

        patient: "Patient"
        lab_value: "LabValue"
        unit: "Unit"
        sample: "PatientLabSample"

    @classmethod
    def create_lab_value_by_sample(
        cls, sample:"PatientLabSample", lab_value_name:str, 
        value:Optional[float]=None, value_str:Optional[str]=None, 
        unit:Optional["Unit"]=None
    ):
        from ..laboratory import LabValue
        patient = sample.patient
        lab_value = LabValue.objects.get(name=lab_value_name)


        pat_lab_val = cls.objects.create(
            patient = patient,
            lab_value = lab_value,
            value = value,
            value_str = value_str,
            sample = sample,
            unit = unit,
        )

        pat_lab_val.save()

        return pat_lab_val

    def __str__(self):
        formatted_datetime = self.datetime.strftime('%Y-%m-%d %H:%M')
        # normal_range = self.get_normal_range()
        norm_range_string = f'[{self.normal_range.get("min", "")} - {self.normal_range.get("max", "")}]'
        _str = f'{self.lab_value} - {self.value} {self.unit} - {norm_range_string} ({formatted_datetime})'
        return _str
    
    def get_normal_range(self):
        lab_value = self.lab_value
        patient = self.patient

        age = patient.age()
        gender = patient.gender

        normal_range_dict = lab_value.get_normal_range(
            age,gender
        )
        return normal_range_dict

    
    def set_min_norm_value(self, value, save = True):
        self.normal_range["min"] = value
        if save:
            self.save()

    def set_max_norm_value(self, value, save = True):
        self.normal_range["max"] = value
        if save:
            self.save()

    def set_norm_values_from_default(self):

        normal_range_dict = self.get_normal_range()
        self.set_min_norm_value(normal_range_dict["min"], save = False)
        self.set_max_norm_value(normal_range_dict["max"], save = False)
        self.save()

    def set_unit_from_default(self):
        self.unit = self.lab_value.default_unit
        self.save()

    def get_value(self):
        if self.value:
            return self.value
        else:
            return self.value_str
        
    def get_value_field_name(self):
        if self.value:
            return "value"
        else:
            return "value_str"
        
    # customize save method so that if a numeric value exists, we round it to the precision of the lab value
    def save(self, *args, **kwargs):
        if self.value:
            precision = self.lab_value.numeric_precision
            self.value = round(self.value, precision)
        super().save(*args, **kwargs)

    def set_value_by_distribution(self, distribution=None, save = True):
        from ...administration.person.patient import Patient
        from ..laboratory import LabValue
        from ...other.distribution import (
            DateValueDistribution,
            SingleCategoricalValueDistribution,
            NumericValueDistribution,
            MultipleCategoricalValueDistribution,
        ) 
        import warnings  

        patient:Patient = self.patient

        lab_value:LabValue = self.lab_value

        assert self.lab_value, "Lab value must be set to set value by distribution"
        self.unit = self.lab_value.default_unit

        if not distribution:
            distribution = lab_value.get_default_default_distribution()

            if not distribution:
                warnings.warn(
                    f"No distribution set for lab value {lab_value}, assuming uniform numeric distribution based on normal values"
                )

                if not self.normal_range.get("min", None) or not self.normal_range.get("max", None):
                    self.set_norm_values_from_default()

                self.normal_range:dict
                _min = self.normal_range.get("min", 0.0001)
                _max = self.normal_range.get("max", 100)
                _name = "auto-" + self.lab_value.name + "-distribution-default-uniform" 
                distribution = NumericValueDistribution(
                    name = _name,
                    min_descriptor = _min,
                    max_max_desciptor = _max,
                    distribution_type = "uniform"
                )

                value = distribution.generate_value(lab_value=lab_value, patient=patient)
                self.value = value
                if save:
                    self.save()
                
                return value

        if isinstance(distribution, SingleCategoricalValueDistribution):
            value = distribution.generate_value()
            self.value_str = value
            if save:
                self.save()
            return value
        
        elif isinstance(distribution, NumericValueDistribution):
            value = distribution.generate_value(
                lab_value=lab_value,
                patient=patient
            )
            self.value = value
            if save:
                self.save()
            return value
        
        elif isinstance(distribution, MultipleCategoricalValueDistribution):
            value = distribution.generate_value()
            self.value_str = value
            if save:
                self.save()
            return value

        elif isinstance(distribution, DateValueDistribution):
            # raise not implemented error
            value = distribution.generate_value()
            self.value = value
            if save:
                self.save()
            return value

    @property
    def links(self) -> "RequirementLinks":
        """
        Aggregates and returns all related model instances relevant for requirement evaluation
        as a RequirementLinks object.
        """
        from endoreg_db.utils.links.requirement_link import RequirementLinks

        return RequirementLinks(
            patient_lab_values=[self] # Include the lab value itself
        )





# See Pipe 1 video file function.
#

"""
Management command to import a video file into the database.
This command is designed to be run from the command line and takes various arguments
to specify the video file, center name, and other options.
"""

from django.core.management import BaseCommand
from django.core.files.base import ContentFile
from django.db import connection
from pathlib import Path
from endoreg_db.models import VideoFile
from endoreg_db.models.administration.center import Center
from endoreg_db.models.medical.hardware import EndoscopyProcessor
# #FIXME
# from endoreg_db.management.commands import validate_video

from endoreg_db.utils.video.ffmpeg_wrapper import check_ffmpeg_availability # ADDED

import logging
logger = logging.getLogger(__name__)

# Import frame cleaning functionality - simplified approach
FRAME_CLEANING_AVAILABLE = False

# Try to import lx_anonymizer using the existing working import method from create_from_file
try:
    # Check if we can find the lx-anonymizer directory
    current_file = Path(__file__)
    endoreg_db_root = current_file.parent.parent.parent.parent
    lx_anonymizer_path = endoreg_db_root / "lx-anonymizer"
    
    if lx_anonymizer_path.exists():
        # Add to Python path temporarily
        import sys
        if str(lx_anonymizer_path) not in sys.path:
            sys.path.insert(0, str(lx_anonymizer_path))
        
        # Try simple import
        from lx_anonymizer import FrameCleaner, ReportReader
        
        FRAME_CLEANING_AVAILABLE = True
        logger.debug("Successfully imported lx_anonymizer modules")
        
        # Remove from path to avoid conflicts
        if str(lx_anonymizer_path) in sys.path:
            sys.path.remove(str(lx_anonymizer_path))
            
except Exception as e:
    logger.debug(f"Frame cleaning not available: {e}")
    FRAME_CLEANING_AVAILABLE = False

IMPORT_MODELS = [
    VideoFile.__name__,
]

IMPORT_METADATA = {
    VideoFile.__name__: {
        "uuid": VideoFile.uuid,
        "raw_file": VideoFile.raw_file,
        "processed_file": VideoFile.processed_file,
        "foreign_keys": [],
        "foreign_key_models": [],
    },
}

class Command(BaseCommand):
    help = """
        Creates a new VideoFile object in the database.
        1. Validates the existence of the specified center and processor
        2. Checks that the video file is saved and anonymized
        3. Creates or updates a ModelMeta entry with the specified parameters
    """

    def add_arguments(self, parser):
        
        """
        Adds command-line arguments for the video import management command.
        
        Defines options for specifying the video file path, associated center and processor names, directory roots for frames and videos, deletion and saving behavior, model path, segmentation usage, and verbosity.
        """
        parser.add_argument(
            "--verbose",
            action="store_true",
            help="Display verbose output",
        )
        parser.add_argument(
            "--center_name",
            type=str,
            default="university_hospital_wuerzburg",
            help="Name of the center to associate with video",
        )
        
        # add the path to the video file
        parser.add_argument(
            "video_file",
            type=Path,
            help="Path to the video file to import",
        )

        # frame dir parent
        parser.add_argument(
            "--frame_dir_root",
            type=str,
            default="~/test-data/raw_frame_dir",
            help="Path to the frame directory",
        )

        # video dir
        parser.add_argument(
            "--video_dir_root",
            type=str,
            default="~/test-data/raw_video_dir",
            help="Path to the video directory",
        )

        # delete source
        parser.add_argument(
            "--delete_source",
            action="store_true",
            default=False,
            help="Delete the source video file after importing",
        )

        # save video file
        parser.add_argument(
            "--save_video_file",
            action="store_true",
            default=True,
            help="Save the video file to the video directory",
        )

        # model_path
        parser.add_argument(
            "--model_name",
            type=str,
            default="image_multilabel_classification_colonoscopy_default",
            help="AiModel Name",
        )
        
        # 
        parser.add_argument(
            "--segmentation",
            action="store_true",
            help="Whether to use segmentation",
        )
        
        parser.add_argument(
            "--processor_name",
            type=str,
            default="olympus_cv_1500",
            help="Name of the processor to associate with video",
        )

    def handle(self, *args, **options):  
        
        """
        Imports a video file into the database, associating it with a specified medical center and endoscopy processor, and optionally applies AI-based segmentation.
        
        Checks for required dependencies, loads reference data, validates the existence of the specified center and processor, and processes the video file. If segmentation is enabled, retrieves the latest segmentation model metadata. Handles interactive processor selection if multiple are available, creates a new `VideoFile` entry, and invokes the processing pipeline. Can optionally delete the source file or save the video to a specified directory. Reports the outcome of the import and processing steps.
        """
        try: # ADDED
            check_ffmpeg_availability() # ADDED
            self.stdout.write(self.style.SUCCESS("FFMPEG is available")) # ADDED
        except FileNotFoundError as e: # ADDED
            self.stderr.write(self.style.ERROR(str(e))) # ADDED
            # Decide if the command should exit or if FFMPEG is optional for some operations
            # For this command, it seems FFMPEG is critical for VideoFile.pipe_1 and VideoFile.create_from_file
            return # ADDED
        
        self.stdout.write(f"Current database: {connection.alias}")
        self.stdout.write(self.style.SUCCESS("Starting video import..."))      

        # Should not be invoked here but in a previous db setup step
        # load_gender_data()
        # load_disease_data()
        # load_event_data()
        # load_information_source()
        # load_examination_data()
        # load_center_data()
        # load_endoscope_data()

        segmentation = options["segmentation"]

        verbose = options["verbose"]
        center_name = options["center_name"]
        video_file = options["video_file"]
        frame_dir_root = options["frame_dir_root"]
        video_dir_root = options["video_dir_root"]
        delete_source = options["delete_source"]
        save_video_file = options["save_video_file"]
        model_name = options["model_name"]
        processor_name = options["processor_name"]
        video_file = Path(video_file).expanduser()

        
        assert isinstance(delete_source, bool), "delete_source must be a boolean"
        assert isinstance(save_video_file, bool), "save_video_file must be a boolean"
        assert isinstance(verbose, bool), "verbose must be a boolean"
        assert isinstance(center_name, str), "center_name must be a string"
        assert isinstance(video_file, Path), "video_file must be a Path"
        assert isinstance(frame_dir_root, str), "frame_dir_root must be a string"
                # Assert Center exists -> Does not exist methods are deprecated
        try:
            center = Center.objects.get(name=center_name)
            self.stdout.write(self.style.SUCCESS(f"Using center: {center.name_en}"))
        except Center.DoesNotExist:
            self.stdout.write(self.style.ERROR(f"Center not found: {center_name}"))
            return

        # Assert Processor Exists
        if processor_name is None:
            processors_qs = center.endoscopy_processors.all()
            proc_count = processors_qs.count()
            if proc_count == 0:
                raise AssertionError(
                    f"No processors linked to '{center.name}' and no processor called '{processor_name}' exists. "
                    "Fallback from default Processor applied."
                )
            elif proc_count == 1:
                processor = processors_qs.first()
            else:

                processor = self._choose_processor_interactively(processors_qs)
                self.stdout.write(self.style.SUCCESS(f"Using processor: {processor.name}"))
        
        else:
            processor = EndoscopyProcessor.objects.get(name=processor_name)

            cns = processor.centers.values_list("name", flat=True)
            if center_name not in cns:
                self.stdout.write(
                    self.style.ERROR(
                        f"Processor '{processor_name}' is not linked to center '{center_name}'."
                    )
                )
                return

        if not Path(video_file).exists():
            self.stdout.write(self.style.ERROR(f"Video file not found: {video_file} saving unsuccessful."))
            return AssertionError(f"Video file not found: {video_file}")
        
        # Create VideoFile instance first
        video_file_obj = VideoFile.create_from_file_initialized(
            file_path=video_file,
            center_name=center_name,
            processor_name=processor_name,
            delete_source=delete_source,
            save_video_file=save_video_file, # Add this line
        )
        
        if not video_file_obj:
            self.stdout.write(self.style.ERROR("Failed to create VideoFile instance"))
            return
        
        self.stdout.write(self.style.SUCCESS(f"Created VideoFile with UUID: {video_file_obj.uuid}"))
        
        # Initialize video specs and frames
        video_file_obj.initialize_video_specs()
        video_file_obj.initialize_frames()
        
        # Run Pipe 1 for OCR and AI processing
        self.stdout.write(self.style.SUCCESS("Starting Pipe 1 processing (OCR + AI)..."))
        success = video_file_obj.pipe_1(
            model_name=model_name,
            delete_frames_after=True,
            ocr_frame_fraction=0.01,
            ocr_cap=5
        )
        
        if not success:
            self.stdout.write(self.style.ERROR("Pipe 1 processing failed"))
            return
            
        self.stdout.write(self.style.SUCCESS("Pipe 1 processing completed"))
        
        # Ensure minimum patient data is available
        self.stdout.write(self.style.SUCCESS("Ensuring minimum patient data..."))
        self._ensure_default_patient_data(video_file_obj)
        
        # Frame-level anonymization integration
        if FRAME_CLEANING_AVAILABLE and video_file_obj.raw_file:
            try:
                self.stdout.write(self.style.SUCCESS("Starting frame-level anonymization..."))
                # Properly instantiate FrameCleaner and ReportReader with correct arguments
                frame_cleaner = FrameCleaner()
                report_reader = ReportReader(
                    report_root_path=str(video_file_obj.raw_file.path),
                    locale="de_DE",  # Default German locale for medical data
                    text_date_format="%d.%m.%Y"  # Common German date format
                )
                
                # Updated to handle new return signature (path, metadata)
                cleaned_video_path, extracted_metadata = frame_cleaner.clean_video(
                    video_path=Path(video_file_obj.raw_file.path),
                    endoscope_roi=video_file_obj.processor.get_roi_endoscope_image() if video_file_obj.processor else None,
                    endoscope_data_roi_nested=video_file_obj.processor.get_rois() if video_file_obj.processor else None,
                    output_path=video_file_obj.get_processed_file_path(),
                    technique="mask_overlay"  # Use mask overlay technique as default, if not set this will be inferred.
                )
                
                # Save the cleaned video using Django's FileField
                with open(cleaned_video_path, 'rb') as f:
                    video_file_obj.raw_file.save(cleaned_video_path.name, ContentFile(f.read()))
                video_file_obj.save()
                
                self.stdout.write(self.style.SUCCESS(f"Frame cleaning completed: {cleaned_video_path.name}"))
                self.stdout.write(self.style.SUCCESS(f"Extracted metadata: {extracted_metadata}"))
                
            except Exception as e:
                self.stdout.write(self.style.WARNING(f"Frame cleaning failed, continuing with original video: {e}"))
        elif not FRAME_CLEANING_AVAILABLE:
            self.stdout.write(self.style.WARNING("Frame cleaning not available (lx_anonymizer not found)"))
        
        # Now call pipe_1 on the VideoFile instance
        if segmentation:
            success = video_file_obj.pipe_1(model_name=model_name, model_meta_version = None)

            if success:
                self.stdout.write(self.style.SUCCESS("Pipeline 1 completed successfully"))
            else:
                self.stdout.write(self.style.ERROR("Pipeline 1 failed"))
    
    def _ensure_default_patient_data(self, video_file):
        """
        Ensure video has minimum required patient data in SensitiveMeta.
        Creates default values if data is missing after OCR processing.
        """
        from endoreg_db.models import SensitiveMeta
        from datetime import date
        
        if not video_file.sensitive_meta:
            self.stdout.write(self.style.WARNING(f"No SensitiveMeta found for video {video_file.uuid}, creating default"))
            
            # Create default SensitiveMeta with placeholder data
            default_data = {
                "patient_first_name": "Patient",
                "patient_last_name": "Unknown", 
                "patient_dob": date(1990, 1, 1),  # Default DOB
                "examination_date": date.today(),
                "center_name": video_file.center.name if video_file.center else "university_hospital_wuerzburg"
            }
            
            try:
                sensitive_meta = SensitiveMeta.create_from_dict(default_data)
                video_file.sensitive_meta = sensitive_meta
                video_file.save(update_fields=['sensitive_meta'])
                self.stdout.write(self.style.SUCCESS(f"Created default SensitiveMeta for video {video_file.uuid}"))
            except Exception as e:
                self.stdout.write(self.style.ERROR(f"Failed to create default SensitiveMeta for video {video_file.uuid}: {e}"))
                
        else:
            # Update existing SensitiveMeta with missing fields
            update_needed = False
            update_data = {}
            
            if not video_file.sensitive_meta.patient_first_name:
                update_data["patient_first_name"] = "Patient"
                update_needed = True
                
            if not video_file.sensitive_meta.patient_last_name:
                update_data["patient_last_name"] = "Unknown"
                update_needed = True
                
            if not video_file.sensitive_meta.patient_dob:
                update_data["patient_dob"] = date(1990, 1, 1)
                update_needed = True
                
            if not video_file.sensitive_meta.examination_date:
                update_data["examination_date"] = date.today()
                update_needed = True
                
            if update_needed:
                try:
                    video_file.sensitive_meta.update_from_dict(update_data)
                    self.stdout.write(self.style.SUCCESS(f"Updated missing SensitiveMeta fields for video {video_file.uuid}: {list(update_data.keys())}"))
                except Exception as e:
                    self.stdout.write(self.style.ERROR(f"Failed to update SensitiveMeta for video {video_file.uuid}: {e}"))
            else:
                self.stdout.write(self.style.SUCCESS(f"SensitiveMeta for video {video_file.uuid} already has all required fields"))
    
    def _choose_processor_interactively(
        self, processors_qs
    ) -> EndoscopyProcessor:
        """
        Interactively prompts the user to select an endoscopy processor from a list.
        
        Displays all available processors and requests user input until a valid selection is made. Aborts the process if the user interrupts input.
        
        Args:
            processors_qs: Queryset of EndoscopyProcessor objects to present for selection.
        
        Returns:
            The EndoscopyProcessor object chosen by the user.
        """
        # turn the QS into a concrete list so we can index it later
        processors = list(processors_qs)           # -> [EndoscopyProcessor, …]

        self.stdout.write(self.style.ERROR(
            f"\nThe centre has {len(processors)} endoscopy processors.\n"
            "Choose one for this import:\n"
        ))
        for idx, proc in enumerate(processors, 1):
            self.stdout.write(f"  [{idx}] {proc.name}")

        while True:                                # keep prompting until valid
            try:
                choice = input("Processor number › ").strip()
            except (EOFError, KeyboardInterrupt):
                self.stderr.write("\nAborted.")
                raise SystemExit(1)

            try:
                index = int(choice) - 1
                if not 0 <= index < len(processors):
                    raise ValueError
            except ValueError:
                self.stderr.write("❌  Please enter a number in the list above.\n")
                continue

            return processors[index]               # ← the chosen object


from rest_framework import serializers
import logging

from ...models import SensitiveMeta

logger = logging.getLogger(__name__)

class SensitiveMetaDetailSerializer(serializers.ModelSerializer):
    """
    Serializer for displaying SensitiveMeta details with verification state.
    Includes all relevant fields for annotation and verification.
    """
    
    # State verification fields
    is_verified = serializers.SerializerMethodField()
    dob_verified = serializers.SerializerMethodField()
    names_verified = serializers.SerializerMethodField()
    
    # Related fields for better display
    center_name = serializers.CharField(source="center.name", read_only=True)
    patient_gender_name = serializers.CharField(source="patient_gender.name", read_only=True)
    
    # Examiner information
    examiners_display = serializers.SerializerMethodField()
    
    # Formatted dates for display
    patient_dob_display = serializers.SerializerMethodField()
    examination_date_display = serializers.SerializerMethodField()
    
    # Hash displays (last 8 characters for security)
    patient_hash_display = serializers.SerializerMethodField()
    examination_hash_display = serializers.SerializerMethodField()

    class Meta:
        model = SensitiveMeta
        fields = [
            'id',
            'patient_first_name',
            'patient_last_name', 
            'patient_dob',
            'patient_dob_display',
            'examination_date',
            'examination_date_display',
            'center_name',
            'patient_gender_name',
            'endoscope_type',
            'endoscope_sn',
            'patient_hash_display',
            'examination_hash_display',
            'examiners_display',
            'is_verified',
            'dob_verified',
            'names_verified',
        ]
        read_only_fields = [
            'id',
            'patient_hash_display',
            'examination_hash_display',
        ]

    def get_is_verified(self, obj):
        """
        Return the overall verification status of the given SensitiveMeta instance.
        
        Returns:
            bool: True if the instance is verified; False if the verification attribute is missing.
        
        Raises:
            Exception: Propagates unexpected exceptions after logging.
        """
        try:
            return obj.is_verified
        except AttributeError:
            return False
        except Exception as e:
            logger.exception(f"Unexpected error in get_is_verified for SensitiveMeta {getattr(obj, 'pk', None)}: {e}")
            raise

    def get_dob_verified(self, obj):
        """
        Return the date of birth verification status for the given object.
        
        Returns:
            bool: True if the date of birth is verified; otherwise, False if unavailable or on error.
        """
        try:
            return obj.state.dob_verified
        except Exception:
            return False

    def get_names_verified(self, obj):
        """
        Return whether the patient's names have been verified.
        
        Returns:
            bool: True if the names are verified; False if not verified or if an error occurs.
        """
        try:
            return obj.state.names_verified
        except Exception:
            return False

    def get_examiners_display(self, obj):
        """
        Return a list of examiner full names for the given SensitiveMeta instance.
        
        Returns:
            list[str]: List of examiner names in "First Last" format, or an empty list if unavailable.
        """
        try:
            if obj.pk:
                examiners = obj.examiners.all()
                return [f"{e.first_name} {e.last_name}" for e in examiners]
            return []
        except Exception as e:
            logger.warning(f"Error getting examiners for SensitiveMeta {obj.pk}: {e}")
            return []

    def get_patient_dob_display(self, obj):
        """
        Return the patient's date of birth formatted as "YYYY-MM-DD" for display.
        
        Returns:
            str or None: Formatted date string if available, otherwise None.
        """
        if obj.patient_dob:
            return obj.patient_dob.strftime("%Y-%m-%d")
        return None

    def get_examination_date_display(self, obj):
        """
        Return the examination date formatted as "YYYY-MM-DD" for display.
        
        Returns:
            str or None: Formatted examination date string, or None if not set.
        """
        if obj.examination_date:
            return obj.examination_date.strftime("%Y-%m-%d")
        return None

    def get_patient_hash_display(self, obj):
        """
        Return the last eight characters of the patient's hash, prefixed with ellipsis, or None if not available.
        
        Returns:
            str or None: Truncated patient hash for display, or None if the hash is not set.
        """
        if obj.patient_hash:
            return f"...{obj.patient_hash[-8:]}"
        return None

    def get_examination_hash_display(self, obj):
        """
        Return the last eight characters of the examination hash, prefixed with ellipsis, or None if not available.
        
        Returns:
            str or None: Truncated examination hash for display, or None if the hash is not set.
        """
        if obj.examination_hash:
            return f"...{obj.examination_hash[-8:]}"
        return None




from rest_framework import serializers
import logging

logger = logging.getLogger(__name__)


class LabelVideoSegmentAnnotationSerializer(serializers.Serializer):
    """
    Serializer for annotation data.
    Handles segment annotations with metadata including segmentId.
    """
    id = serializers.IntegerField(read_only=True)
    videoId = serializers.IntegerField(required=True)
    startTime = serializers.FloatField(required=False)
    endTime = serializers.FloatField(required=False)
    type = serializers.CharField(max_length=50, required=True)
    text = serializers.CharField(max_length=1000, required=False, allow_blank=True)
    tags = serializers.ListField(
        child=serializers.CharField(max_length=100),
        required=False,
        default=list
    )
    userId = serializers.CharField(max_length=100, required=False)
    isPublic = serializers.BooleanField(default=False)
    confidence = serializers.FloatField(required=False, min_value=0.0, max_value=1.0)
    metadata = serializers.DictField(required=False, default=dict)
    
    def validate(self, data):
        """
        Validates annotation data, enforcing required fields and value constraints for segment annotations.
        
        For annotations of type 'segment', ensures both `startTime` and `endTime` are present and that `startTime` is less than `endTime`. If `metadata.segmentId` is provided, verifies it is a valid integer.
        
        Raises:
            serializers.ValidationError: If validation fails.
            
        Returns:
            dict: The validated annotation data.
        """
        annotation_type = data.get('type')
        
        if annotation_type == 'segment':
            if 'startTime' not in data or 'endTime' not in data:
                raise serializers.ValidationError(
                    "Segment annotations must include startTime and endTime"
                )
            
            start_time = data.get('startTime')
            end_time = data.get('endTime')
            
            if start_time >= end_time:
                raise serializers.ValidationError(
                    "startTime must be less than endTime"
                )
                
            # Validate metadata contains segmentId if provided
            metadata = data.get('metadata', {})
            segment_id = metadata.get('segmentId')
            if segment_id is not None:
                try:
                    int(segment_id)
                except (ValueError, TypeError):
                    raise serializers.ValidationError(
                        "metadata.segmentId must be a valid integer"
                    )
        
        return data
    
    #@maxhild we need to verify if this is the right place for these methods, if so we should implement them here
    def create(self, validated_data):
        """
        Assigns a unique ID to the annotation data and returns the updated data dictionary.
        
        In this simulated implementation, the method mimics creating a new annotation by incrementing an internal counter for the ID. No data is persisted to a database or external storage.
         
        Returns:
            dict: The annotation data with an assigned unique ID.
        """
        # Simulate creating an annotation with an ID
        validated_data['id'] = getattr(self, '_next_id', 1)
        self._next_id = validated_data['id'] + 1
        return validated_data
    
    def update(self, instance, validated_data):
        """
        Update annotation data structure.
        """
        for attr, value in validated_data.items():
            instance[attr] = value
        return instance
    
    def save(self, **kwargs):
        """
        Save the annotation.
        """
        if self.instance:
            return self.update(self.instance, self.validated_data)
        else:
            return self.create(self.validated_data)


from typing import Literal
from endoreg_db.models import VideoFile


from rest_framework import serializers


class VideoFileListSerializer(serializers.ModelSerializer):
    """
    Minimal serializer to return only basic video information
    for the video selection dropdown in Vue.js.
    """

    # Add computed fields for video status
    status = serializers.SerializerMethodField()
    assignedUser = serializers.SerializerMethodField()
    anonymized = serializers.SerializerMethodField()

    class Meta:
        model = VideoFile
        fields = ["id", "original_file_name", "status", "assignedUser", "anonymized"]

    def get_status(self, obj:VideoFile) -> Literal['completed'] | Literal['in_progress'] | Literal['available']:
        """
        Determine the processing status of a video file as 'completed', 'in_progress', or 'available'.
        
        Returns:
            str: The video's status based on its processing state.
            
        Raises:
            ValueError: If the video's state cannot be accessed.
        """
        try:
            state = obj.state
            if not state:
                return 'available'
            if state.anonymized:
                return 'completed'
            if state.frames_extracted:
                return 'in_progress'
            return 'available'
        except Exception as _e:
            raise ValueError("Video state should not be None") from _e

    def get_assignedUser(self, obj):
        """
        Returns the user assigned to the video, or None if no user is assigned.
        
        Currently always returns None as user assignment is not implemented.
        """
        # For now return None, can be extended when user assignment is implemented
        return None

    def get_anonymized(self, obj):
        """
        Determine whether the video has been anonymized.
        
        Returns:
            bool: True if the video's state indicates it is anonymized; False otherwise or if state information is unavailable.
        """
        try:
            state = getattr(obj, 'state', None)
            if state:
                return getattr(state, 'anonymized', False)
            return False
        except Exception:
            return False


from django.urls import path

from endoreg_db.views import (
    ExaminationStatsView,
    VideoSegmentStatsView,
    SensitiveMetaStatsView,
    GeneralStatsView,
)

url_patterns = [
    # ---------------------------------------------------------------------------------------
    # STATISTICS API ENDPOINTS
    #
    # Diese Endpunkte stellen Dashboard-Statistiken bereit für das Frontend
    # ---------------------------------------------------------------------------------------

    # Examination Statistics API
    # GET /api/examinations/stats/
    # Liefert Statistiken über Examinations und PatientExaminations
    path('examinations/stats/', ExaminationStatsView.as_view(), name='examination_stats'),

    # Video Segment Statistics API  
    # GET /api/video-segment/stats/  (Note: singular 'segment' to match frontend)
    # Liefert Statistiken über Video-Segmente und Label-Verteilung
    path('video-segment/stats/', VideoSegmentStatsView.as_view(), name='video_segment_stats'),

    # Alternative Video Segments Statistics API (plural version for compatibility)
    # GET /api/video-segments/stats/
    path('video-segments/stats/', VideoSegmentStatsView.as_view(), name='video_segments_stats'),

    # Sensitive Meta Statistics API
    # GET /api/video/sensitivemeta/stats/
    # Liefert Statistiken über SensitiveMeta-Einträge und Verifikationsstatus
    path('video/sensitivemeta/stats/', SensitiveMetaStatsView.as_view(), name='sensitive_meta_stats'),

    # General Dashboard Statistics API
    # GET /api/stats/
    # Liefert allgemeine Übersichtsstatistiken für das Dashboard
    path('stats/', GeneralStatsView.as_view(), name='general_stats'),

    path(
    'video-segments/stats/', 
    VideoSegmentStatsView.as_view(), 
    name='video_segments_stats'
    ),
]


# endoreg_db/services/anonymization.py
from pathlib import Path
from django.db import transaction
from django.conf import settings
from endoreg_db.models import VideoFile, RawPdfFile
from endoreg_db.services.video_import import VideoImportService
from endoreg_db.services.pdf_import import PdfImportService
from endoreg_db.utils.paths import STORAGE_DIR
import logging

logger = logging.getLogger(__name__)

class AnonymizationService:
    """
    Orchestrates long‑running anonymization tasks so the view only
    does HTTP <-> Service translation.
    """
    
    def __init__(self, project_root: Path = None):
        """
        Initialize the AnonymizationService with service instances.
        
        Args:
            project_root: Path to the project root. If None, uses settings.BASE_DIR
        """
        if project_root is None:
            project_root = STORAGE_DIR
        
        self.video_service = VideoImportService()
        self.pdf_service = PdfImportService()

    # ---------- READ ----------------------------------------------------
    @staticmethod
    def get_status(file_id: int):
        """
        Retrieve the anonymization status and media type for a file by its ID.
        
        Returns:
            dict or None: A dictionary containing the file's media type and anonymization status if found, or None if no matching file exists.
        """
        vf = VideoFile.objects.select_related("state", "sensitive_meta").filter(pk=file_id).first()
        if vf:
            return {
                "mediaType": "video",
                "anonymizationStatus": vf.state.anonymization_status if vf.state else "not_started",
                "fileExists": bool(vf.raw_file and hasattr(vf.raw_file, 'path')),
                "uuid": str(vf.uuid) if vf.uuid else None,
            }

        pdf = RawPdfFile.objects.select_related("state", "sensitive_meta").filter(pk=file_id).first()
        if pdf:
            return {
                "mediaType": "pdf",
                "anonymizationStatus": pdf.state.anonymization_status if pdf.state else "not_started",
                "fileExists": bool(pdf.file and hasattr(pdf.file, 'path')),
                "hash": pdf.pdf_hash,
            }
        return None

    # ---------- COMMANDS ------------------------------------------------
    @transaction.atomic
    def start(self, file_id: int):
        """
        Start anonymization process for a file by its ID.
        
        Args:
            file_id: The ID of the file to anonymize
            
        Returns:
            str or None: Media type if successful, None if file not found
        """
        # Try VideoFile first
        vf = VideoFile.objects.select_related("state", "sensitive_meta", "center", "video_meta__processor").filter(pk=file_id).first()
        if vf:
            try:
                logger.info(f"Starting video anonymization for VideoFile ID: {file_id}")
                
                # Check if already processed
                if vf.state and vf.state.anonymized:
                    logger.info(f"VideoFile {file_id} already anonymized, skipping")
                    return "video"
                
                # Get file path
                file_path = vf.get_raw_file_path()
                if not file_path or not Path(file_path).exists():
                    logger.error(f"Raw file not found for VideoFile {file_id}: {file_path}")
                    return None
                
                # Get processor name
                processor_name = None
                if vf.video_meta and vf.video_meta.processor:
                    processor_name = vf.video_meta.processor.name
                elif hasattr(vf, 'processor') and vf.processor:
                    processor_name = vf.processor.name
                
                # Get center name
                center_name = vf.center.name if vf.center else "unknown_center"
                
                # Mark as started
                if vf.state:
                    vf.state.processing_started = True
                    vf.state.save(update_fields=["processing_started"])
                
                # Use VideoImportService for anonymization
                self.video_service.import_and_anonymize(
                    file_path=file_path,
                    center_name=center_name,
                    processor_name=processor_name,
                    save_video=True,
                    delete_source=False
                )
                
                logger.info(f"Video anonymization completed for VideoFile ID: {file_id}")
                return "video"
                
            except Exception as e:
                logger.error(f"Failed to anonymize VideoFile {file_id}: {e}")
                # Mark as failed if state exists
                if vf.state:
                    vf.state.processing_started = False  # Mark processing as not started due to failure
                    vf.state.save(update_fields=["processing_started"])
                raise

        # Try RawPdfFile
        pdf = RawPdfFile.objects.select_related("state", "sensitive_meta", "center").filter(pk=file_id).first()
        if pdf:
            try:
                logger.info(f"Starting PDF processing for RawPdfFile ID: {file_id}")
                
                # Check if already processed
                if pdf.state and getattr(pdf.state, 'anonymized', False):
                    logger.info(f"RawPdfFile {file_id} already processed, skipping")
                    return "pdf"
                
                # Get file path
                if not pdf.file or not hasattr(pdf.file, 'path'):
                    logger.error(f"PDF file not found for RawPdfFile {file_id}")
                    return None
                
                file_path = Path(pdf.file.path)
                if not file_path.exists():
                    logger.error(f"PDF file does not exist: {file_path}")
                    return None
                
                # Get center name
                center_name = pdf.center.name if pdf.center else "unknown_center"
                
                # Mark as started
                if pdf.state:
                    pdf.state.processing_started = True
                    pdf.state.save(update_fields=["processing_started"])
                elif pdf.sensitive_meta:
                    pdf.sensitive_meta.anonymization_started = True
                    pdf.sensitive_meta.save(update_fields=["anonymization_started"])
                
                
                # Use PdfImportService for processing
                # Use PdfImportService for processing
                self.pdf_service.import_and_anonymize(
                    file_path=file_path,
                    center_name=center_name,
                )
                
                logger.info(f"PDF processing completed for RawPdfFile ID: {file_id}")
                return "pdf"
                
            except Exception as e:
                logger.error(f"Failed to process RawPdfFile {file_id}: {e}")
                # Mark as failed if state exists
                if pdf.state:
                    pdf.state.processing_failed = True
                    pdf.state.save(update_fields=["processing_failed"])
                elif pdf.sensitive_meta:
                    pdf.sensitive_meta.processing_failed = True
                    pdf.sensitive_meta.save(update_fields=["processing_failed"])
                raise

        logger.warning(f"No file found with ID: {file_id}")
        return None

    @staticmethod
    @transaction.atomic
    def validate(file_id: int):
        vf = VideoFile.objects.select_related("state").filter(pk=file_id).first()
        if vf:
            vf.state.mark_anonymization_validated()
            return "video"

        pdf = RawPdfFile.objects.select_related("state").filter(pk=file_id).first()
        if pdf:
            pdf.state.mark_anonymization_validated()
            return "pdf"

        return None
    
    def list_items():
        video_files = VideoFile.objects.select_related("state").all()
        pdf_files = RawPdfFile.objects.select_related("state").all()  # was sensitive_meta

        data = []
        for vf in video_files:
            data.append({
                "id": vf.id,
                "mediaType": "video",
                "anonymizationStatus": vf.state.anonymization_status if vf.state else "not_started",
                "createdAt": vf.date_created,
                "updatedAt": vf.date_modified,
            })
            
            

        for pdf in pdf_files:
            data.append({
                "id": pdf.id,
                "mediaType": "pdf",
                "anonymizationStatus": pdf.state.anonymization_status if pdf.state else "not_started",
                "createdAt": pdf.date_created,
                "updatedAt": pdf.date_modified,
            })
        return data


        return data



"""
Management command to check and configure authentication settings.
"""

from django.core.management.base import BaseCommand
from django.conf import settings
from rest_framework.permissions import IsAuthenticated, AllowAny
import os


class Command(BaseCommand):
    help = 'Check and configure authentication settings based on environment'

    def add_arguments(self, parser):
        parser.add_argument(
            '--mode',
            choices=['check', 'dev', 'prod'],
            default='check',
            help='Mode: check current settings, set dev mode, or set prod mode'
        )
        parser.add_argument(
            '--show-permissions',
            action='store_true',
            help='Show which permission classes will be used'
        )

    def handle(self, *args, **options):
        mode = options['mode']
        
        if mode == 'check':
            self.check_current_settings()
        elif mode == 'dev':
            self.set_dev_mode()
        elif mode == 'prod':
            self.set_prod_mode()
            
        if options['show_permissions']:
            self.show_permission_classes()

    def check_current_settings(self):
        """Check current authentication configuration."""
        debug_mode = getattr(settings, 'DEBUG', False)
        settings_module = os.environ.get('DJANGO_SETTINGS_MODULE', 'Unknown')
        
        self.stdout.write("\n" + "="*50)
        self.stdout.write("🔍 AUTHENTICATION CONFIGURATION CHECK")
        self.stdout.write("="*50)
        
        self.stdout.write(f"Settings Module: {settings_module}")
        self.stdout.write(f"DEBUG Mode: {debug_mode}")
        
        if debug_mode:
            self.stdout.write(self.style.WARNING("🔓 Authentication: DISABLED (AllowAny)"))
            self.stdout.write("   - All API endpoints are accessible without login")
            self.stdout.write("   - Suitable for development and testing")
        else:
            self.stdout.write(self.style.SUCCESS("🔒 Authentication: ENABLED (IsAuthenticated)"))
            self.stdout.write("   - API endpoints require valid authentication")
            self.stdout.write("   - Suitable for production deployment")
            
        # Check if Keycloak is configured
        keycloak_server = getattr(settings, 'KEYCLOAK_SERVER_URL', None)
        if keycloak_server:
            self.stdout.write(f"Keycloak Server: {keycloak_server}")
        else:
            self.stdout.write(self.style.WARNING("⚠️  Keycloak not configured"))
            
        self.stdout.write("="*50 + "\n")

    def set_dev_mode(self):
        """Instructions for setting development mode."""
        self.stdout.write(self.style.SUCCESS("\n🛠️  DEVELOPMENT MODE SETUP"))
        self.stdout.write("To enable development mode with disabled authentication:")
        self.stdout.write("")
        self.stdout.write("1. Use dev settings:")
        self.stdout.write("   export DJANGO_SETTINGS_MODULE=dev.dev_settings")
        self.stdout.write("   python manage.py runserver")
        self.stdout.write("")
        self.stdout.write("2. Or run with explicit settings:")
        self.stdout.write("   python manage.py runserver --settings=dev.dev_settings")
        self.stdout.write("")
        self.stdout.write("This will set DEBUG=True and disable authentication requirements.")

    def set_prod_mode(self):
        """Instructions for setting production mode."""
        self.stdout.write(self.style.SUCCESS("\n🏭 PRODUCTION MODE SETUP"))
        self.stdout.write("To enable production mode with authentication:")
        self.stdout.write("")
        self.stdout.write("1. Use production settings:")
        self.stdout.write("   export DJANGO_SETTINGS_MODULE=prod_settings")
        self.stdout.write("   python manage.py runserver --settings=prod_settings")
        self.stdout.write("")
        self.stdout.write("2. Set required environment variables:")
        self.stdout.write("   export DJANGO_SECRET_KEY='your-secure-secret-key'")
        self.stdout.write("   export KEYCLOAK_CLIENT_SECRET='your-keycloak-secret'")
        self.stdout.write("")
        self.stdout.write("This will set DEBUG=False and enable authentication requirements.")

    def show_permission_classes(self):
        """Show which permission classes are being used."""
        debug_mode = getattr(settings, 'DEBUG', False)
        
        self.stdout.write("\n" + "="*40)
        self.stdout.write("🔐 PERMISSION CLASSES IN USE")
        self.stdout.write("="*40)
        
        if debug_mode:
            permission_class = "AllowAny"
            icon = "🔓"
        else:
            permission_class = "IsAuthenticated"
            icon = "🔒"
            
        self.stdout.write(f"{icon} Current Mode: {permission_class}")
        self.stdout.write("")
        self.stdout.write("Views using dynamic permissions:")
        self.stdout.write("  • video_segments_view")
        self.stdout.write("  • VideoViewSet")
        self.stdout.write("  • Other API endpoints")
        self.stdout.write("")
        self.stdout.write("Views always requiring authentication:")
        self.stdout.write("  • video_segment_detail_view")
        self.stdout.write("  • video_segments_by_label_id_view")
        self.stdout.write("  • video_segments_by_label_name_view")
        self.stdout.write("="*40 + "\n")


from typing import TYPE_CHECKING
from django.core.exceptions import ObjectDoesNotExist
from rest_framework import serializers
from endoreg_db.models import (
    Label,
    VideoFile,
    LabelVideoSegment,
    InformationSource
)

import logging
logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    from endoreg_db.serializers import LabelVideoSegmentSerializer

def _get_video_file(self, video_id):
    """
    Retrieve a VideoFile instance by its ID.
    
    Raises a serializers.ValidationError if the VideoFile does not exist.
    """
    try:
        return VideoFile.objects.get(id=video_id)
    except ObjectDoesNotExist as exc:
        raise serializers.ValidationError(f"VideoFile with id {video_id} does not exist") from exc

def _get_label(self, label_id, label_name):
    """
    Retrieve a Label instance by ID or name, creating it if necessary.
    
    Raises a ValidationError if the label cannot be found, created, or if neither identifier is provided.
    """
    if label_id:
        try:
            return Label.objects.get(id=label_id)
        except ObjectDoesNotExist as exc:
            raise serializers.ValidationError(f"Label with id {label_id} does not exist") from exc
    elif label_name:
        label, _ = Label.get_or_create_from_name(label_name)
        if not label:
            raise serializers.ValidationError(f"Failed to create or retrieve label with name {label_name}")
        return label
    else:
        raise serializers.ValidationError("Either label_id or label_name must be provided")

def _validate_fps(self, video_file):
    """
    Validate and return the frames per second (FPS) value from a video file.
    
    Raises a validation error if the FPS is missing, not a positive number, or cannot be converted to a float.
    
    Returns:
        float: The validated FPS value.
    """
    fps_raw = video_file.get_fps()
    if fps_raw is None:
        raise serializers.ValidationError("Video file must have a defined FPS")
    try:
        fps = float(fps_raw)
        if fps <= 0:
            raise ValueError("FPS must be a positive number")
    except (ValueError, TypeError):
        raise serializers.ValidationError("Invalid FPS format in video file")
    return fps

def _calculate_frame_numbers(self, validated_data, fps):
    """
    Convert start and end times in the input data to corresponding frame numbers using the provided FPS value.
    
    Parameters:
        validated_data (dict): Input data containing 'start_time' and 'end_time' keys.
        fps (float): Frames per second of the video.
    
    Returns:
        dict: The input data dictionary updated with 'start_frame_number' and 'end_frame_number' keys.
    
    Raises:
        serializers.ValidationError: If frame numbers cannot be determined from the provided data.
    """
    if validated_data.get('start_time') is not None:
        validated_data['start_frame_number'] = int(float(validated_data['start_time']) * fps)
    if validated_data.get('end_time') is not None:
        validated_data['end_frame_number'] = int(float(validated_data['end_time']) * fps)
    if 'start_frame_number' not in validated_data or 'end_frame_number' not in validated_data:
        raise serializers.ValidationError("Could not determine frame numbers from provided data")
    return validated_data

def _get_information_source(self):
    """
    Retrieve or create the information source for manual annotation.
    
    Returns:
        InformationSource: The source object representing manual annotation.
    """
    source, _ = InformationSource.objects.get_or_create(
        name='Manual Annotation',
        defaults={
            'description': 'Manually created label segments via web interface',
        }
    )
    return source

def _create(self:"LabelVideoSegmentSerializer", validated_data):
    """
    Create and persist a new LabelVideoSegment instance from validated input data.
    
    This method extracts relevant fields from the input, retrieves or creates related objects (video file, label, information source), validates video FPS, calculates frame numbers from time values, and creates a new LabelVideoSegment. Raises a validation error if required data is missing or invalid, or if creation fails.
    
    Returns:
        LabelVideoSegment: The newly created and saved segment instance.
    """
    try:
        # Extract convenience fields
        video_id = validated_data.pop('video_id')
        label_id = validated_data.pop('label_id', None)
        label_name = validated_data.pop('label_name', None)
        start_time = validated_data.pop('start_time', None)
        end_time = validated_data.pop('end_time', None)

        video_file = self.get_video_file(video_id)
        label = self.get_label(label_id, label_name)
        fps = self.validate_fps(video_file)
        validated_data['start_time'] = start_time
        validated_data['end_time'] = end_time
        validated_data = self.calculate_frame_numbers(validated_data, fps)
        source = self.get_information_source()

        # Create the segment directly
        segment = LabelVideoSegment.safe_create(
            video_file=video_file,
            label=label,
            source=source,
            start_frame_number=validated_data['start_frame_number'],
            end_frame_number=validated_data['end_frame_number'],
            prediction_meta=None  # No prediction meta for manual annotations
        )
        segment.save()

        logger.info("Created new video segment: %s for video %s with label %s",
                    segment.pk, video_id, label.name if label else 'None')
        return segment

    except serializers.ValidationError as e:
        logger.error("Validation error while creating video segment: %s", str(e))
        raise
    except Exception as e:
        logger.error("Unexpected error while creating video segment: %s", str(e))
        raise serializers.ValidationError(f"Failed to create segment: {str(e)}")



"""
Django management command to initialize the default AI model with metadata.
This command ensures that a default AI model exists with proper ModelMeta records.
"""

from django.core.management.base import BaseCommand
from django.core.management import call_command
from pathlib import Path
#FIXME
from endoreg_db.data.ai_model_meta import (
    default_multilabel_classification
)

from endoreg_db.models import AiModel
from endoreg_db.helpers.data_loader import (
    load_ai_model_label_data,
    load_ai_model_data,
)


class Command(BaseCommand):
    help = """
    Initialize the default AI model with metadata.
    This command ensures that the default segmentation model exists with proper ModelMeta records.
    """

    def add_arguments(self, parser):
        """
        Adds the --force command-line argument to control metadata recreation.
        
        The --force flag, when specified, forces the recreation of model metadata even if it already exists.
        """
        parser.add_argument(
            '--force',
            action='store_true',
            help='Force recreation of the model metadata even if it exists',
        )

    def handle(self, *args, **options):
        """
        Initializes the default AI model and its metadata, creating or updating as needed.
        
        Loads required AI model label and model data, ensures the presence of a default AI model, and creates associated model metadata if it does not already exist or if the `--force` flag is specified. Generates a dummy weights file if necessary and invokes the metadata creation command. Verifies successful creation and outputs status messages throughout the process.
        """
        force = options.get('force', False)
        default_multilabel_classification()
        # First ensure the basic AI model data is loaded
        self.stdout.write("Loading AI model label data...")
        load_ai_model_label_data()
        
        self.stdout.write("Loading AI model data...")
        load_ai_model_data()

        # Check if default model exists
        default_model_name = "image_multilabel_classification_colonoscopy_default"
        
        try:
            ai_model = AiModel.objects.get(name=default_model_name)
            self.stdout.write(f"Found AI model: {ai_model.name}")
        except AiModel.DoesNotExist:
            self.stdout.write(
                self.style.ERROR(f"AI model '{default_model_name}' not found. Make sure AI model data is loaded.")
            )
            return

        # Check if model metadata exists
        existing_meta = ai_model.metadata_versions.first()
        if existing_meta and not force:
            self.stdout.write(
                self.style.SUCCESS(f"Model metadata already exists for {ai_model.name}. Use --force to recreate.")
            )
            return

        # Create default model metadata
        self.stdout.write("Creating default model metadata...")
        
        # Use a dummy weights file path for now - in production this should point to actual model weights
        dummy_weights_path = Path(__file__).parent.parent.parent / "assets" / "dummy_model.ckpt"
        
        # Create the dummy weights file if it doesn't exist
        dummy_weights_path.parent.mkdir(parents=True, exist_ok=True)
        if not dummy_weights_path.exists():
            dummy_weights_path.write_bytes(b"dummy weights content")
            self.stdout.write(f"Created dummy weights file at {dummy_weights_path}")

        try:
            # Create ModelMeta using the create_multilabel_model_meta command
            call_command(
                "create_multilabel_model_meta",
                "--model_path", str(dummy_weights_path),
                "--model_name", default_model_name,
                "--image_classification_labelset_name", "multilabel_classification_colonoscopy",
                "--activation_function_name", "sigmoid",
                "--model_meta_version", "1",
            )
            
            self.stdout.write(
                self.style.SUCCESS(f"Successfully created model metadata for {default_model_name}")
            )
            
            # Verify the model can be retrieved
            from endoreg_db.helpers.default_objects import get_latest_segmentation_model
            model_meta = get_latest_segmentation_model()
            self.stdout.write(
                self.style.SUCCESS(f"Verified: Model metadata can be retrieved: {model_meta}")
            )
            
        except Exception as e:
            self.stdout.write(
                self.style.ERROR(f"Failed to create model metadata: {e}")
            )
            raise


# Django model for the report reader flag
# have name and value
# name is natural key

from django.db import models

class ReportReaderFlagManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)
    
class ReportReaderFlag(models.Model):
    objects = ReportReaderFlagManager()
    name = models.CharField(max_length=255, unique=True)
    value = models.CharField(max_length=255)
    
    def natural_key(self):
        return (self.name,)
    
    def __str__(self):
        return self.name


import shutil
from pathlib import Path
from endoreg_db.utils import DbConfig

TEMPLATE_DIR = Path("./conf_template")
CONF_DIR = Path("./conf")

DB_CFG_PATH = TEMPLATE_DIR / "db.yaml"

CONF_TARGETS = {
    "root": CONF_DIR,
    "db": CONF_DIR / "db.yaml",
}


def main(conf_dir: Path = CONF_TARGETS["root"], template_dir: Path = TEMPLATE_DIR):
    db_template = template_dir / "db.yaml"
    assert db_template.exists(), f"Missing Template {DB_CFG_PATH}"

    if not conf_dir.exists():
        conf_dir.mkdir()

    if not CONF_TARGETS["db"].exists():
        db_cfg = DbConfig.from_file(DB_CFG_PATH)
        db_cfg.custom_validate()
        db_cfg.to_file(CONF_TARGETS["db"], ask_override=True)


if __name__ == "__main__":
    main()



# AI
from .ai import (
    AiModel,
    ActiveModel,
    ModelType,
)

# Case
from .case import (
    Case,
    CaseTemplate,
    CaseTemplateRule,
    CaseTemplateRuleType,
    CaseTemplateRuleValue,
    CaseTemplateRuleValueType,
    CaseTemplateType,
)

# Center
from .center import (
    Center,
    CenterProduct,
    CenterResource,
    CenterWaste,
    CenterShift,
)


#TODO Review module
# from .permissions import () 

# Person
from .person import (
    Person,
    Examiner, 
    Patient, 
    PortalUserInfo,     
    FirstName,
    LastName,
    Profession,
    Employee,
    EmployeeType,
    EmployeeQualification,
)

# Product
from .product import (
    Product,
    ProductMaterial,
    ProductGroup,
    ReferenceProduct,
    ProductWeight,
)

from .qualification import (
    Qualification,
    QualificationType,
)

from .shift import (
    Shift,
    ShiftType,
    ScheduledDays,
)

__all__ = [
    # AI
    "AiModel",
    "ActiveModel",
    "ModelType",

    # Case
    "Case",
    "CaseTemplate",
    "CaseTemplateRule",
    "CaseTemplateRuleType",
    "CaseTemplateRuleValue",
    "CaseTemplateRuleValueType",
    "CaseTemplateType",

    # Center
    "Center",
    "CenterProduct",
    "CenterResource",
    "CenterWaste",
    "CenterShift",

    # Person
    "Person",
    "Patient",
    "Examiner",
    "PortalUserInfo",
    "FirstName",
    "LastName",
    "Profession",
    "Employee",
    "EmployeeType",
    "EmployeeQualification",

    # Product
    'Product',
    'ProductMaterial',
    'ProductGroup',
    'ReferenceProduct',
    'ProductWeight',

    # Qualification
    "Qualification",
    "QualificationType",

    # Shift
    "Shift",
    "ShiftType",
    "ScheduledDays",
]




from faker import Faker
from datetime import datetime, timedelta
import random
import re
from .custom_logger import get_logger

def replace_umlauts(text):
    """Replace German umlauts with their corresponding letter (ä -> ae)."""
    text = text.replace('ä', 'ae')
    text = text.replace('ö', 'oe')
    text = text.replace('ü', 'ue')
    text = text.replace('Ä', 'AE')
    text = text.replace('Ö', 'OE')
    text = text.replace('Ü', 'UE')
    return text

def replace_large_numbers(text):
    """Replaces all numbers with at least 5 digits with random numbers of the same length."""
    def random_number(n):
        return ''.join([str(random.randint(0, 9)) for _ in range(n)])
    
    numbers_to_replace = re.findall(r'\b\d{5,}\b', text)
    
    for number in numbers_to_replace:
        length = len(number)
        random_num = random_number(length)
        text = text.replace(number, random_num)

    return text

def remove_titles(name):
    """Remove titles like 'Dr.' and 'Dr. med.' from names."""
    return re.sub(r'(Dr\. med\. |Dr\. |Prof\.)', '', name)

def cutoff_leading_text(text, cutoff_flags):
    """
    Remove all text above the first occurrence of any cutoff flag.
    Used primarily for removing letterheads in medical reports.
    """
    if not cutoff_flags:  # If no flags provided, return text as is
        return text
        
    lines = text.split('\n')
    for i, line in enumerate(lines):
        for flag in cutoff_flags:
            if flag in line:
                return '\n'.join(lines[i:])
    return text

def cutoff_trailing_text(text, cutoff_flags):
    """
    Remove all text below the first occurrence of any cutoff flag.
    Used primarily for removing footers in medical reports.
    """
    if not cutoff_flags:  # If no flags provided, return text as is
        return text
        
    lines = text.split('\n')
    for i, line in enumerate(lines):
        for flag in cutoff_flags:
            if flag in line:
                return '\n'.join(lines[:i+1])
    return text

def replace_employee_names(text, first_names, last_names, locale=None):
    """Replace known employee names with fake names."""
    fake = Faker(locale=locale)
    if first_names:
        for first_name in first_names:
            if first_name and first_name in text:
                text = text.replace(first_name, fake.first_name())
    
    if last_names:
        for last_name in last_names:
            if last_name and last_name in text:
                text = text.replace(last_name, fake.last_name())
                
    return text

def anonymize_text(text, report_meta, text_date_format="%d.%m.%Y", 
                   lower_cut_off_flags=None, upper_cut_off_flags=None, 
                   locale="de_DE", first_names=None, last_names=None,
                   apply_cutoffs=False, verbose=False):
    """
    Anonymizes the given text by replacing personal information.
    
    Args:
        text (str): The text to anonymize.
        report_meta (dict): Metadata containing patient and examiner information.
        text_date_format (str): Format for dates in the text.
        lower_cut_off_flags (list): Markers to remove text after a given line.
        upper_cut_off_flags (list): Markers to remove text before a given line.
        locale (str): Locale for generating fake data.
        first_names (list): List of first names to use.
        last_names (list): List of last names to use.
        apply_cutoffs (bool): Whether to apply text cutoffs.
        verbose (bool): Enable verbose logging.
        
    Returns:
        str: Anonymized text.
    """
    logger = get_logger(__name__, verbose=verbose)
    logger.info("Starting text anonymization")
    fake = Faker(locale=locale)
    anonymized_text = text

    # Ersetze Patientennamen, falls Schlüssel in report_meta anders benannt sind, 
    # nehmen wir hier z.B. patient_first_name und patient_last_name:
    if report_meta.get("patient_first_name") and report_meta.get("patient_last_name"):
        first_name = report_meta["patient_first_name"]
        last_name = report_meta["patient_last_name"]
        pseudo_first_name = fake.first_name()
        pseudo_last_name = fake.last_name()
        anonymized_text = anonymized_text.replace(first_name, pseudo_first_name)
        anonymized_text = anonymized_text.replace(last_name, pseudo_last_name)
        logger.debug(f"Replaced patient name: {first_name} {last_name} -> {pseudo_first_name} {pseudo_last_name}")
    
    if report_meta.get("examiner_first_name") and report_meta.get("examiner_last_name"):
        examiner_first_name = report_meta["examiner_first_name"]
        examiner_last_name = report_meta["examiner_last_name"]
        pseudo_examiner_first_name = fake.first_name()
        pseudo_examiner_last_name = fake.last_name()
        anonymized_text = anonymized_text.replace(examiner_first_name, pseudo_examiner_first_name)
        anonymized_text = anonymized_text.replace(examiner_last_name, pseudo_examiner_last_name)
        logger.debug(f"Replaced examiner name: {examiner_first_name} {examiner_last_name} -> {pseudo_examiner_first_name} {pseudo_examiner_last_name}")

    # Verbessertes Cut-Off: Zuerst oberen Teil entfernen, dann unteren.
    if apply_cutoffs:
        logger.info("Applying text cutoffs")
        # Entferne den oberen Teil (alles vor Erleben eines Upper‑Flags)
        if upper_cut_off_flags:
            anonymized_text = cutoff_leading_text(anonymized_text, upper_cut_off_flags)
            logger.debug("Applied upper cutoff")
        # Entferne den unteren Teil (alles nach Erleben eines Lower‑Flags)
        if lower_cut_off_flags:
            anonymized_text = cutoff_trailing_text(anonymized_text, lower_cut_off_flags)
            logger.debug("Applied lower cutoff")
    
    logger.info("Text anonymization complete")
    return anonymized_text




from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from endoreg_db.models import Label, VideoPredictionMeta, VideoFile


def _create_from_video(
    cls,
    source: "VideoFile",
    prediction_meta: Optional["VideoPredictionMeta"],
    label: Optional["Label"],
    start_frame_number: int,
    end_frame_number: int,
):
    """
    Create a LabelVideoSegment instance from a VideoFile.
    """
    from endoreg_db.models import VideoFile

    if not isinstance(source, VideoFile):
        raise ValueError("Source must be a VideoFile instance.")

    if start_frame_number < 0 or end_frame_number < 0:
        raise ValueError(
            f"Frame numbers must be non-negative: start={start_frame_number}, end={end_frame_number}"
        )

    if start_frame_number > end_frame_number:
        raise ValueError(
            f"Start frame number ({start_frame_number}) must be less than or equal to "
            f"end frame number ({end_frame_number})"
        )

    segment = cls(
        start_frame_number=start_frame_number,
        end_frame_number=end_frame_number,
        label=label,
        video_file=source,
        prediction_meta=prediction_meta,
    )
    return segment



from django.db import models

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from endoreg_db.models import (
        Employee,
        Qualification,
    )

class EmployeeQualification(models.Model):
    """
    Model representing an employee's qualification.
    """
    employee = models.OneToOneField(
        "Employee",
        on_delete=models.CASCADE,
        related_name="qualification",
    )
    qualifications = models.ManyToManyField(
        "Qualification",
        related_name="employee_qualifications",
    )

    if TYPE_CHECKING:
        employee: models.ForeignKey["Employee"]
        qualifications: models.QuerySet["Qualification"]

    def __str__(self):
        """
        Returns a human-readable string summarizing the employee and their qualifications.
        
        If the employee has qualifications, lists them separated by commas; otherwise, indicates that no qualifications are assigned.
        """
        qualification_list = self.qualifications.all()
        if qualification_list:
            # Assuming Qualification model has a decent __str__ representation
            qual_names = ", ".join(str(q) for q in qualification_list)
            return f"{self.employee} - Qualifications: {qual_names}"
        return f"{self.employee} - No qualifications assigned"


import huggingface_hub
from typing import Optional

def download_segmentation_model(
    repo_id: str = "wg-lux/colo_segmentation_RegNetX800MF_base",
    filename: str = "model.safetensors",
    cache_dir: Optional[str] = None
) -> str:
    """
    Downloads a segmentation model from Hugging Face and caches it locally.

    Args:
        repo_id (str): The Hugging Face repository ID (default: wg-lux model).
        filename (str): The specific file to download from the repo (default: model.safetensors).
        cache_dir (str): The directory to cache the downloaded model. If None, uses HF default cache.

    Returns:
        str: The local path to the downloaded model.
        
    Example:
        >>> model_path = download_segmentation_model()
        >>> # Downloads from wg-lux/colo_segmentation_RegNetX800MF_base
    """
    local_path = huggingface_hub.hf_hub_download(
        repo_id=repo_id,
        filename=filename,
        cache_dir=cache_dir,
        force_download=False,
        resume_download=True,
    )
    return local_path


from .report_list import ReportListSerializer
from .report import ReportDataSerializer
from .secure_file_url import SecureFileUrlSerializer

__all__ = [
    "ReportListSerializer",
    "ReportDataSerializer",
    "SecureFileUrlSerializer"
]


from .administration import (
    ActiveModelSerializer,
    AiModelSerializer,
    CenterSerializer,
    GenderSerializer,
    ModelTypeSerializer,
)
from .examination import (
    ExaminationDropdownSerializer,
    ExaminationSerializer,
    ExaminationTypeSerializer,
)
from .finding import FindingSerializer
from .finding_classification import (
    FindingClassificationChoiceSerializer,
    FindingClassificationSerializer,
)
from .label import ImageClassificationAnnotationSerializer, LabelSerializer
from .label_video_segment import (
    LabelVideoSegmentAnnotationSerializer,
    LabelVideoSegmentSerializer,
)
from .meta import (
    PDFFileForMetaSerializer,
    ReportMetaSerializer,
    SensitiveMetaDetailSerializer,
    SensitiveMetaUpdateSerializer,
    SensitiveMetaVerificationSerializer,
    VideoMetaSerializer,
)
from .misc import (
    FileOverviewSerializer,
    StatsSerializer,
    TranslatableFieldMixin,
    UploadCreateResponseSerializer,
    UploadJobStatusSerializer,
    VoPPatientDataSerializer,
)
from .patient import PatientDropdownSerializer, PatientSerializer
from .patient_examination import PatientExaminationSerializer
from .patient_finding import (
    PatientFindingClassificationSerializer,
    PatientFindingDetailSerializer,
    PatientFindingInterventionSerializer,
    PatientFindingListSerializer,
    PatientFindingSerializer,
    PatientFindingWriteSerializer,
)
from .pdf import RawPdfAnonyTextSerializer
from .report import ReportDataSerializer, ReportListSerializer, SecureFileUrlSerializer
from .video.video_metadata import VideoMetadataSerializer
from .video.video_processing_history import VideoProcessingHistorySerializer
from .video_examination import (
    VideoExaminationCreateSerializer,
    VideoExaminationSerializer,
    VideoExaminationUpdateSerializer,
)

__all__ = [
    # Administration
    "CenterSerializer",
    "GenderSerializer",
    "ActiveModelSerializer",
    "ModelTypeSerializer",
    "AiModelSerializer",
    # Examination
    "ExaminationSerializer",
    "ExaminationTypeSerializer",
    "ExaminationDropdownSerializer",
    # Finding
    "FindingSerializer",
    "FindingClassificationSerializer",
    "FindingClassificationChoiceSerializer",
    "LabelSerializer",
    "ImageClassificationAnnotationSerializer",
    # LabelVideoSegment
    "LabelVideoSegmentSerializer",
    "LabelVideoSegmentAnnotationSerializer",
    # Meta
    "PDFFileForMetaSerializer",
    "ReportMetaSerializer",
    "SensitiveMetaDetailSerializer",
    "SensitiveMetaUpdateSerializer",
    "SensitiveMetaVerificationSerializer",
    "VideoMetaSerializer",
    # Misc
    "FileOverviewSerializer",
    "VoPPatientDataSerializer",
    "StatsSerializer",
    "UploadJobStatusSerializer",
    "UploadCreateResponseSerializer",
    "TranslatableFieldMixin",
    # Patient
    "PatientSerializer",
    "PatientDropdownSerializer",
    # Patient Examination
    "PatientExaminationSerializer",
    # Patient Finding
    "PatientFindingSerializer",
    "PatientFindingClassificationSerializer",
    "PatientFindingDetailSerializer",
    "PatientFindingInterventionSerializer",
    "PatientFindingListSerializer",
    "PatientFindingWriteSerializer",
    # PDF
    "RawPdfAnonyTextSerializer",
    # Report
    "ReportListSerializer",
    "ReportDataSerializer",
    "SecureFileUrlSerializer",
    # Video Correction (Phase 1.1)
    "VideoMetadataSerializer",
    "VideoProcessingHistorySerializer",
    # Video Examination
    "VideoExaminationSerializer",
    "VideoExaminationCreateSerializer",
    "VideoExaminationUpdateSerializer",
]



"""
Django management command to validate video file existence and accessibility.
"""
from django.core.management.base import BaseCommand

from endoreg_db.models import VideoFile
from pathlib import Path
import logging

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Validate video file existence and accessibility'

    def add_arguments(self, parser):
        parser.add_argument(
            '--video-id',
            type=int,
            help='Check specific video ID',
        )
        parser.add_argument(
            '--verbose',
            action='store_true',
            help='Enable verbose output',
        )
        parser.add_argument(
            '--fix-missing',
            action='store_true',
            help='Mark videos with missing files as inactive',
        )

    def handle(self, *args, **options):
        """Validate video files and their accessibility."""
        verbose = options['verbose']
        video_id = options.get('video_id')
        fix_missing = options['fix_missing']

        if verbose:
            self.stdout.write(self.style.SUCCESS("Starting video validation..."))

        # Query videos
        if video_id:
            try:
                videos = [VideoFile.objects.get(pk=video_id)]
                self.stdout.write(f"Checking specific video ID: {video_id}")
            except VideoFile.DoesNotExist:
                self.stdout.write(self.style.ERROR(f"Video with ID {video_id} not found"))
                return
        else:
            videos = VideoFile.objects.all()
            self.stdout.write(f"Checking {videos.count()} videos...")

        missing_files = []
        accessible_files = []
        corrupted_files = []

        for video in videos:
            video_status = self.check_video_file(video, verbose)
            
            if video_status['status'] == 'missing':
                missing_files.append(video_status)
            elif video_status['status'] == 'corrupted':
                corrupted_files.append(video_status)
            else:
                accessible_files.append(video_status)

        # Report results
        self.stdout.write("\n" + "="*60)
        self.stdout.write(self.style.SUCCESS("VALIDATION COMPLETE"))
        self.stdout.write("="*60)
        
        self.stdout.write(f"✅ Accessible videos: {len(accessible_files)}")
        self.stdout.write(f"❌ Missing files: {len(missing_files)}")
        self.stdout.write(f"⚠️  Potentially corrupted: {len(corrupted_files)}")

        if missing_files:
            self.stdout.write(self.style.WARNING("\nMISSING FILES:"))
            for file_info in missing_files:
                self.stdout.write(f"  - Video ID {file_info['video_id']}: {file_info['error']}")
                if fix_missing:
                    self.stdout.write("    → Marking as inactive (if applicable)")

        if corrupted_files:
            self.stdout.write(self.style.WARNING("\nPOTENTIALLY CORRUPTED FILES:"))
            for file_info in corrupted_files:
                self.stdout.write(f"  - Video ID {file_info['video_id']}: {file_info['error']}")

        if verbose and accessible_files:
            self.stdout.write(self.style.SUCCESS("\nACCESSIBLE FILES:"))
            for file_info in accessible_files[:10]:  # Show first 10
                self.stdout.write(f"  ✅ Video ID {file_info['video_id']}: {file_info['path']} ({file_info['size_mb']:.1f} MB)")
            
            if len(accessible_files) > 10:
                self.stdout.write(f"  ... and {len(accessible_files) - 10} more")

    def check_video_file(self, video, verbose=False):
        """
        Check a single video file for existence and basic accessibility.
        
        Returns:
            dict: Status information about the video file
        """
        video_info = {
            'video_id': video.id,
            'video_uuid': str(video.uuid) if hasattr(video, 'uuid') else 'N/A',
            'status': 'unknown',
            'path': None,
            'size_mb': 0,
            'error': None
        }

        # Helper to check a file attribute
        def _check_file_attr(obj, attr, path_getter=None, label=None):
            if not hasattr(obj, attr):
                return None
            file_field = getattr(obj, attr)
            if not file_field:
                return None
            try:
                file_path = Path(path_getter(file_field) if path_getter else file_field)
                info = video_info.copy()
                info['path'] = str(file_path)
                if not file_path.exists():
                    info['status'] = 'missing'
                    info['error'] = f"{label or attr.replace('_', ' ').title()} does not exist: {file_path}"
                    return info
                file_size = file_path.stat().st_size
                info['size_mb'] = file_size / (1024 * 1024)
                if file_size == 0:
                    info['status'] = 'corrupted'
                    info['error'] = f"{label or attr.replace('_', ' ').title()} exists but has zero size"
                else:
                    info['status'] = 'accessible'
                return info
            except (ValueError, OSError) as e:
                info = video_info.copy()
                info['path'] = str(getattr(file_field, 'path', file_field))
                info['status'] = 'corrupted'
                info['error'] = f"Cannot access {label or attr.replace('_', ' ').title()}: {e}"
                return info

        # Try each file attribute in order of preference
        result = None
        # active_file_path: direct path string
        result = _check_file_attr(video, 'active_file_path', label='Active file path')
        if result: return result
        # active_file: Django FileField
        result = _check_file_attr(video, 'active_file', path_getter=lambda f: f.path, label='Active file')
        if result: return result
        # raw_file: Django FileField
        result = _check_file_attr(video, 'raw_file', path_getter=lambda f: f.path, label='Raw file')
        if result: return result
        # processed_file: Django FileField
        result = _check_file_attr(video, 'processed_file', path_getter=lambda f: f.path, label='Processed file')
        if result: return result

        # If none found
        video_info['status'] = 'missing'
        video_info['error'] = "No video file paths found (no active_file, raw_file, or processed_file)"
        return video_info


from django.db import models
from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from .patient import PatientDisease


class DiseaseManager(models.Manager):
    """Manager for Disease with natural key support."""

    def get_by_natural_key(self, name):
        """
        Retrieve a Disease instance by its natural key (name).

        Args:
            name: The natural key value used to match the model's 'name' field.

        Returns:
            The Disease instance corresponding to the provided natural key.
        """
        return self.get(name=name)


class Disease(models.Model):
    """
    Represents a specific disease or medical condition.

    Can define associated subcategories and numerical descriptors applicable to the disease itself.
    """
    name = models.CharField(max_length=255, unique=True)
    subcategories = models.JSONField(default=dict)
    numerical_descriptors = models.JSONField(default=dict)

    objects = DiseaseManager()

    if TYPE_CHECKING:
        disease_classifications: models.QuerySet["DiseaseClassification"]
        patient_diseases: models.QuerySet["PatientDisease"]

    def natural_key(self):
        """Returns the natural key (name) as a tuple."""
        return (self.name,)

    def __str__(self):
        """Returns the name of the disease."""
        return str(self.name)

    def get_classifications(self) -> List["DiseaseClassification"]:
        """
        Retrieves all classification systems associated with this disease.

        Returns:
            List[DiseaseClassification]: A list of related disease classification objects.
        """
        classifications: List[DiseaseClassification] = [
            _ for _ in self.disease_classifications.all()
        ]
        return classifications


class DiseaseClassificationManager(models.Manager):
    """Manager for DiseaseClassification with natural key support."""

    def get_by_natural_key(self, name):
        """
        Retrieve a DiseaseClassification instance by its natural key (name).

        Args:
            name: A unique identifier representing the natural key of the instance.

        Returns:
            The DiseaseClassification instance corresponding to the given natural key.
        """
        return self.get(name=name)


class DiseaseClassification(models.Model):
    """
    Represents a classification system applicable to a specific disease (e.g., Forrest classification for ulcers).
    """
    name = models.CharField(max_length=255, unique=True)

    disease = models.ForeignKey(
        Disease, on_delete=models.CASCADE, related_name="disease_classifications"
    )

    objects = DiseaseClassificationManager()

    if TYPE_CHECKING:
        disease: "Disease"
        disease_classification_choices: models.QuerySet["DiseaseClassificationChoice"]

    def natural_key(self):
        """Returns the natural key (name) as a tuple."""
        return (self.name,)

    def __str__(self):
        """Returns the name of the classification."""
        return str(self.name)

    def get_choices(self) -> List["DiseaseClassificationChoice"]:
        """
        Retrieves all choices within this classification system.

        Returns:
            List[DiseaseClassificationChoice]: A list of related disease classification choices.
        """
        choices: List[DiseaseClassificationChoice] = [
            _ for _ in self.disease_classification_choices.all()
        ]
        return choices


class DiseaseClassificationChoiceManager(models.Manager):
    """Manager for DiseaseClassificationChoice with natural key support."""

    def get_by_natural_key(self, name):
        """
        Retrieve a DiseaseClassificationChoice instance by its natural key (name).

        Queries for and returns the instance whose 'name' attribute matches the provided key.

        Args:
            name: The natural key value used to match the model's 'name' field.

        Returns:
            The DiseaseClassificationChoice instance corresponding to the provided natural key.
        """
        return self.get(name=name)


class DiseaseClassificationChoice(models.Model):
    """
    Represents a specific choice within a disease classification system (e.g., Forrest IIa).
    """
    name = models.CharField(max_length=255, unique=True)

    disease_classification = models.ForeignKey(
        DiseaseClassification,
        on_delete=models.CASCADE,
        related_name="disease_classification_choices",
    )

    objects = DiseaseClassificationChoiceManager()

    if TYPE_CHECKING:
        disease_classification: "DiseaseClassification"
        patient_diseases: models.QuerySet["PatientDisease"]

    def natural_key(self):
        """Returns the natural key (name) as a tuple."""
        return (self.name,)

    def __str__(self):
        """Returns the name of the classification choice."""
        return str(self.name)



from endoreg_db.utils.permissions import EnvironmentAwarePermission


from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView


import os


class VideoDownloadProcessedView(APIView):
    """
    GET /api/video-download-processed/{id}/ - Download processed video result
    """
    permission_classes = [EnvironmentAwarePermission]

    def get(self, request, pk):
        # Remove unused 'video' variable
        output_path = request.query_params.get('path')

        # Define the allowed base directory for processed videos
        processed_base_dir = os.path.abspath(os.getenv("PROCESSED_VIDEO_DIR", "/srv/processed_videos"))
        if not output_path:
            return Response(
                {"error": "Processed file not found"},
                status=status.HTTP_404_NOT_FOUND
            )

        # Resolve the absolute path and check if it's within the allowed directory
        abs_output_path = os.path.abspath(output_path)
        if not abs_output_path.startswith(processed_base_dir + os.sep):
            return Response(
                {"error": "Invalid file path"},
                status=status.HTTP_400_BAD_REQUEST
            )

        if not os.path.exists(abs_output_path):
            return Response(
                {"error": "Processed file not found"},
                status=status.HTTP_404_NOT_FOUND
            )

        try:
            from django.http import FileResponse
            with open(abs_output_path, 'rb') as f:
                response = FileResponse(
                    f,
                    content_type='video/mp4'
                )
                response['Content-Disposition'] = f'attachment; filename="{os.path.basename(abs_output_path)}"'
                return response

        except Exception as e:
            return Response(
                {"error": f"Failed to serve file: {str(e)}"},
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )


from endoreg_db.models import Examination, FindingIntervention


from django.shortcuts import get_object_or_404
from rest_framework.decorators import api_view
from rest_framework.response import Response


@api_view(["GET"])
def get_interventions_for_examination(request, exam_id):
    exam = get_object_or_404(Examination, id=exam_id)
    findings = exam.get_available_findings()
    interventions = FindingIntervention.objects.filter(findings__in=findings).distinct()
    return Response([{"id": i.id, "name": i.name} for i in interventions])


from django.db import models
from typing import TYPE_CHECKING, Optional, List  # Added List

# Added imports for type hints
if TYPE_CHECKING:
    from ...administration.person.patient import Patient
    from ..medication import MedicationIndication, Medication, MedicationIntakeTime
    from ...other.unit import Unit
    from ....utils.links.requirement_link import RequirementLinks # Added RequirementLinks

class PatientMedication(models.Model):
    """
    Represents a specific medication prescribed or taken by a patient.

    Links a patient to a medication, its indication, dosage, intake times, and unit.
    """
    patient = models.ForeignKey("Patient", on_delete=models.CASCADE)
    medication_indication = models.ForeignKey(
        "MedicationIndication", on_delete=models.CASCADE,
        related_name="indication_patient_medications", null=True
    )

    medication = models.ForeignKey(
        'Medication', on_delete=models.CASCADE,
        blank=True,
        related_name='medication_patient_medications'
    )

    intake_times = models.ManyToManyField(
        'MedicationIntakeTime', 
        related_name='intake_time_patient_medications',
        blank=True,
    )

    unit = models.ForeignKey(
        'Unit', on_delete=models.CASCADE,
        null=True, blank=True
    )
    dosage = models.JSONField(
        null=True, blank=True
    )
    active = models.BooleanField(default=True)

    objects = models.Manager()

    if TYPE_CHECKING:  # Added type hints block
        patient: "Patient"
        medication_indication: Optional["MedicationIndication"]
        medication: Optional["Medication"]
        intake_times: models.QuerySet["MedicationIntakeTime"]
        unit: Optional["Unit"]
        dosage: Optional[dict]

    @property
    def links(self) -> "RequirementLinks":
        """
        Returns a RequirementLinks object for this PatientMedication instance.
        This is used during requirement evaluation.
        """
        from ....utils.links.requirement_link import RequirementLinks
        
        meds: List["Medication"] = []
        if self.medication:
            meds.append(self.medication)
        
        indications: List["MedicationIndication"] = []
        if self.medication_indication:
            indications.append(self.medication_indication)

        intake_times_list: List["MedicationIntakeTime"] = list(self.intake_times.all())

        return RequirementLinks(
            medications=meds,
            medication_indications=indications,
            medication_intake_times=intake_times_list,
            patient_medications=[self] # Include self in patient_medications
        )

    class Meta:
        verbose_name = 'Patient Medication'
        verbose_name_plural = 'Patient Medications'

    @classmethod
    def create_by_patient_and_indication(cls, patient, medication_indication):
        """Creates a PatientMedication instance linking a patient and an indication."""
        from ..medication import MedicationIndication
        medication_indication: MedicationIndication
        patient_medication = cls.objects.create(patient=patient, medication_indication=medication_indication)
        patient_medication.save()
        
        return patient_medication


    def __str__(self):
        """Returns a string representation including medication, indication, dosage, and intake times."""
        intake_times = self.intake_times.all()
        out = f"{self.medication} (Indication {self.medication_indication}) - "
        out += f"{self.dosage} - {self.unit} - "

        for intake_time in intake_times:
            out += f"{intake_time} - "

        return out




from django.db import models
from typing import TYPE_CHECKING
# models.py in your main app

if TYPE_CHECKING:
    from ..profession import Profession
    from endoreg_db.models import Examiner
    from django.contrib.auth.models import User


class PortalUserInfo(models.Model):
    user = models.OneToOneField("auth.User", on_delete=models.CASCADE)
    profession = models.ForeignKey(
        'endoreg_db.Profession',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="portal_user_infos",
    )
    works_in_endoscopy = models.BooleanField(blank=True, null=True)
    # Add other fields as needed

    examiner = models.OneToOneField(
        "endoreg_db.Examiner",
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="portal_user_info",
    )

    if TYPE_CHECKING:
        user: "User"
        profession: "Profession"
        examiner: "Examiner"

    def __str__(self):
        return self.user.username



from .custom_logger import get_logger
from pathlib import Path
logger=get_logger(__name__)

def expand_roi(startX, startY, endX, endY, expansion, image_shape):
    """
    Expand the ROI by a certain number of pixels in all directions and ensure it is within image boundaries.

    Parameters:
    startX, startY, endX, endY: int
        The starting and ending coordinates of the ROI.
    expansion: int
        The number of pixels to expand the ROI in all directions.
    image_shape: tuple
        The shape of the image to ensure the expanded ROI is within the bounds.

    Returns:
    tuple
        The expanded ROI coordinates.
    """
    startX = max(0, startX - expansion)
    startY = max(0, startY - expansion)
    endX = min(image_shape[1], endX + expansion)
    endY = min(image_shape[0], endY + expansion)
    logger.debug(f"Expanded ROI to ({startX}, {startY}, {endX}, {endY})")
    return (startX, startY, endX, endY)


from endoreg_db.models import (
    PatientExamination
)

#TODO this should be a list of all models that are used in the requirement evaluation
#TODO All those models must have a "get_requirement_links" property
operator_evaluation_models = [
    PatientExamination,
]


from django.urls import path
from PIL.PdfParser import PdfStream

from endoreg_db.views import VideoStreamView
from endoreg_db.views.media import (
    PdfMediaView,  # Alias to avoid conflict with legacy pdf.PDFMediaView
    VideoMediaView,
    pdf_sensitive_metadata,
    pdf_sensitive_metadata_list,
    pdf_sensitive_metadata_verify,
    sensitive_metadata_list,
    video_segment_detail,
    video_segment_validate,
    video_segments_by_pk,
    video_segments_by_video,
    video_segments_collection,
    video_segments_stats,
    video_segments_validate_bulk,
    video_segments_validation_status,
    video_sensitive_metadata,
    video_sensitive_metadata_verify,
)
from endoreg_db.views.pdf.pdf_stream import PdfStreamView
from endoreg_db.views.pdf.reimport import PdfReimportView
from endoreg_db.views.video.correction import (
    VideoApplyMaskView,
    VideoCorrectionView,
    VideoMetadataView,
    VideoProcessingHistoryView,
    VideoRemoveFramesView,
)
from endoreg_db.views.video.reimport import VideoReimportView

# ---------------------------------------------------------------------------------------
# ANNOTATION API ENDPOINTS
#
# New endpoints for segment annotation management that create user-source segments
# POST /api/annotations/ - Create new annotation (creates user segment if type=segment)
# PATCH /api/annotations/<id>/ - Update annotation (creates user segment if timing/label changed)
# ---------------------------------------------------------------------------------------

# Simplified Meta and Validation Endpoints

urlpatterns = [
    # Video media endpoints
    path("media/videos/", VideoMediaView.as_view(), name="video-list"),
    path(
        "media/videos/<int:pk>/", VideoStreamView.as_view(), name="video-detail-stream"
    ),  # Support ?type= params
    path(
        "media/videos/<int:pk>/details/", VideoMediaView.as_view(), name="video-detail"
    ),  # JSON metadata
    path(
        "media/videos/<int:pk>/stream/", VideoStreamView.as_view(), name="video-stream"
    ),  # Legacy support
    # Video Re-import API endpoint (modern media framework)
    # POST /api/media/videos/<int:pk>/reimport/
    # Re-imports a video file to regenerate metadata when OCR failed or data is incomplete
    path(
        "media/videos/<int:pk>/reimport/",
        VideoReimportView.as_view(),
        name="video-reimport",
    ),
    # ---------------------------------------------------------------------------------------
    # VIDEO CORRECTION API ENDPOINTS (Modern Media Framework - October 14, 2025)
    #
    # All video correction endpoints migrated to unified /api/media/videos/<pk>/ pattern
    # These endpoints enable video correction workflows (Phase 1.1):
    # - Analysis: Detect sensitive frames using MiniCPM-o 2.6 or OCR+LLM
    # - Masking: Apply device-specific masks or custom ROI masks
    # - Frame Removal: Remove sensitive frames from videos
    # - Reprocessing: Re-run entire anonymization pipeline
    # - Metadata: View analysis results
    # - History: Track all correction operations
    # ---------------------------------------------------------------------------------------
    # Video Correction API
    # GET /api/media/videos/video-correction/{id}/ - Get video details for correction
    path(
        "media/videos/video-correction/<int:pk>",
        VideoCorrectionView.as_view(),
        name="video-correction",
    ),
    # Video Metadata API
    # GET /api/media/videos/<int:pk>/metadata/
    # Returns analysis results (sensitive frame count, ratio, frame IDs)
    path(
        "media/videos/<int:pk>/metadata/",
        VideoMetadataView.as_view(),
        name="video-metadata",
    ),
    # Video Processing History API
    # GET /api/media/videos/<int:pk>/processing-history/
    # Returns history of all processing operations (masking, frame removal, analysis)
    path(
        "media/videos/<int:pk>/processing-history/",
        VideoProcessingHistoryView.as_view(),
        name="video-processing-history",
    ),
    # Video Analysis API
    # POST /api/media/videos/<int:pk>/analyze/
    # Analyzes video for sensitive frames using MiniCPM-o 2.6 or OCR+LLM
    # Body: { detection_method: 'minicpm'|'ocr_llm'|'hybrid', sample_interval: 30 }
    # Video Masking API
    # POST /api/media/videos/<int:pk>/apply-mask/
    # Applies device mask or custom ROI mask to video
    # Body: { mask_type: 'device'|'custom', device_name: 'olympus', roi: {...} }
    path(
        "media/videos/<int:pk>/apply-mask/",
        VideoApplyMaskView.as_view(),
        name="video-apply-mask",
    ),
    # Video Frame Removal API
    # POST /api/media/videos/<int:pk>/remove-frames/
    # Removes specified frames from video
    # Body: { frame_list: [10,20,30] OR frame_ranges: '10-20,30' OR detection_method: 'automatic' }
    path(
        "media/videos/<int:pk>/remove-frames/",
        VideoRemoveFramesView.as_view(),
        name="video-remove-frames",
    ),
    # ---------------------------------------------------------------------------------------
    # VIDEO SEGMENT API ENDPOINTS (Modern Media Framework - October 14, 2025)
    #
    # Unified segment management endpoints replacing legacy /api/video-segments/
    # Collection: GET/POST all segments across videos
    # Video-scoped: GET/POST segments for specific video
    # Detail: GET/PATCH/DELETE individual segment
    # ---------------------------------------------------------------------------------------
    # Video Segments Collection API
    # GET/POST /api/media/videos/segments/
    # List all video segments across videos or create new segment
    path(
        "media/videos/segments/",
        video_segments_collection,
        name="video-segments-collection",
    ),
    # Video Segments Stats API
    # GET /api/media/videos/segments/stats/
    # Get statistics about video segments
    path(
        "media/videos/segments/stats/",
        video_segments_stats,
        name="video-segments-stats",
    ),
    # Video-Specific Segments API
    # GET/POST /api/media/videos/<int:pk>/segments/
    # List segments for specific video or create segment for video
    path(
        "media/videos/<int:pk>/segments/",
        video_segments_by_video,
        name="video-segments-by-video",
    ),
    # Segment Detail API
    # GET /api/media/videos/<int:pk>/segments/<int:segment_id>/
    # PATCH /api/media/videos/<int:pk>/segments/<int:segment_id>/
    # DELETE /api/media/videos/<int:pk>/segments/<int:segment_id>/
    # Manages individual segment operations
    path(
        "media/videos/<int:pk>/segments/<int:segment_id>/",
        video_segment_detail,
        name="video-segment-detail",
    ),
    # ---------------------------------------------------------------------------------------
    # VIDEO SEGMENT VALIDATION API ENDPOINTS (Modern Media Framework - October 14, 2025)
    #
    # Unified validation endpoints replacing legacy /api/label-video-segment/*/validate/
    # Single: POST validate individual segment
    # Bulk: POST validate multiple segments
    # Status: GET/POST validation status for all segments
    # ---------------------------------------------------------------------------------------
    # Single Segment Validation API
    # POST /api/media/videos/<int:pk>/segments/<int:segment_id>/validate/
    # Validates a single video segment
    # Body: { "is_validated": true, "notes": "..." }
    path(
        "media/videos/<int:pk>/segments/<int:segment_id>/validate/",
        video_segment_validate,
        name="video-segment-validate",
    ),
    # Bulk Segment Validation API
    # POST /api/media/videos/<int:pk>/segments/validate-bulk/
    # Validates multiple segments at once
    # Body: { "segment_ids": [1,2,3], "is_validated": true, "notes": "..." }
    path(
        "media/videos/<int:pk>/segments/validate-bulk/",
        video_segments_validate_bulk,
        name="video-segments-validate-bulk",
    ),
    # Segment Validation Status API
    # GET /api/media/videos/<int:pk>/segments/validation-status/
    # Returns validation statistics for all segments
    # POST /api/media/videos/<int:pk>/segments/validation-status/
    # Marks all segments (or filtered by label) as validated
    # Body: { "label_name": "polyp", "notes": "..." }
    path(
        "media/videos/<int:pk>/segments/validation-status/",
        video_segments_validation_status,
        name="video-segments-validation-status",
    ),
    # ---------------------------------------------------------------------------------------
    # SENSITIVE METADATA ENDPOINTS (Modern Media Framework)
    # ---------------------------------------------------------------------------------------
    # Video Sensitive Metadata (Resource-Scoped)
    # GET/PATCH /api/media/videos/<pk>/sensitive-metadata/
    # Get or update sensitive patient data for a video
    path(
        "media/videos/<int:pk>/sensitive-metadata/",
        video_sensitive_metadata,
        name="video-sensitive-metadata",
    ),
    # POST /api/media/videos/<pk>/sensitive-metadata/verify/
    # Update verification state (dob_verified, names_verified)
    path(
        "media/videos/<int:pk>/sensitive-metadata/verify/",
        video_sensitive_metadata_verify,
        name="video-sensitive-metadata-verify",
    ),
    # PDF Sensitive Metadata (Resource-Scoped)
    # GET/PATCH /api/media/pdfs/<pk>/sensitive-metadata/
    # Get or update sensitive patient data for a PDF
    path(
        "media/pdfs/<int:pk>/sensitive-metadata/",
        pdf_sensitive_metadata,
        name="pdf-sensitive-metadata",
    ),
    # POST /api/media/pdfs/<pk>/sensitive-metadata/verify/
    # Update verification state (dob_verified, names_verified)
    path(
        "media/pdfs/<int:pk>/sensitive-metadata/verify/",
        pdf_sensitive_metadata_verify,
        name="pdf-sensitive-metadata-verify",
    ),
    # List Endpoints (Collection-Level)
    # GET /api/media/sensitive-metadata/
    # List all sensitive metadata (combined PDFs and Videos)
    # Supports filtering: ?content_type=pdf|video&verified=true&search=name
    path(
        "media/sensitive-metadata/",
        sensitive_metadata_list,
        name="sensitive-metadata-list",
    ),
    # GET /api/media/pdfs/sensitive-metadata/
    # List sensitive metadata for PDFs only
    # Replaces legacy /api/pdf/sensitivemeta/list/
    path(
        "media/pdfs/sensitive-metadata/",
        pdf_sensitive_metadata_list,
        name="pdf-sensitive-metadata-list",
    ),
    # PDF media endpoints
    path("media/pdfs/", PdfMediaView.as_view(), name="pdf-list"),
    path("media/pdfs/<int:pk>/", PdfMediaView.as_view(), name="pdf-detail"),
    path(
        "media/pdfs/<int:pk>/stream/", PdfStreamView.as_view(), name="pdf-stream"
    ),  # Support ?type=raw|anonymized params
    # PDF Re-import API endpoint (modern media framework)
    # POST /api/media/pdfs/<int:pk>/reimport/
    # Re-imports a PDF file to regenerate metadata when OCR failed or data is incomplete
    path(
        "media/pdfs/<int:pk>/reimport/", PdfReimportView.as_view(), name="pdf-reimport"
    ),
]
# ---------------------------------------------------------------------------------------



from django.core.management.base import BaseCommand
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.apps import apps


class Command(BaseCommand):
    help = "Create additional user groups and permissions for all models in 'endoreg_db' app."

    def add_arguments(self, parser):
        parser.add_argument(
            '--verbose',
            action='store_true',
            help='Display verbose output',
        )

    def handle(self, *args, **options):
        verbose = options['verbose']

        # Create groups
        groups = ["demo", "verified", "agl", "endo_reg_user", "g_play_user", "ukw_user"]
        for group_name in groups:
            _group, created = Group.objects.get_or_create(name=group_name)
            if verbose and created:
                self.stdout.write(self.style.SUCCESS(f"Created group {group_name}"))

        if verbose:
            self.stdout.write(self.style.SUCCESS("All groups processed successfully."))



import logging
from typing import TYPE_CHECKING, Optional
from django.db import models
from pathlib import Path
import cv2
import numpy as np
if TYPE_CHECKING:
    from endoreg_db.models import Label, ImageClassificationAnnotation, VideoFile

logger = logging.getLogger(__name__)

# Unified Frame model
class Frame(models.Model):
    video = models.ForeignKey(
        "VideoFile",
        on_delete=models.CASCADE,
        related_name="frames",
        blank=False,
        null=False,
    )
    frame_number = models.PositiveIntegerField()
    relative_path = models.CharField(max_length=512)
    timestamp = models.FloatField(null=True, blank=True)
    is_extracted = models.BooleanField(default=False)

    if TYPE_CHECKING:
        image_classification_annotations: models.QuerySet["ImageClassificationAnnotation"]
        labels: models.QuerySet["Label"]
        video: "VideoFile"
    class Meta:
        unique_together = ('video', 'frame_number')
        ordering = ['video', 'frame_number']

    @property
    def file_path(self) -> Path:
        """
        Return the absolute filesystem path to the frame image by combining the video's frame directory with the frame's relative path.
        
        Returns:
            Path: The absolute path to the frame image file.
        """
        base_dir = self.video.get_frame_dir_path()
        return base_dir / self.relative_path
    
    @property
    def predictions(self) -> models.QuerySet["ImageClassificationAnnotation"]:
        """
        Return all image classification annotations for this frame that are linked to an information source of type "prediction".
        
        Returns:
            QuerySet: A queryset of related ImageClassificationAnnotation objects filtered to those whose information source type is "prediction".
        """
        return self.image_classification_annotations.filter(
            information_source__information_source_types__name="prediction"
        )
    
    @property
    def manual_annotations(self) -> models.QuerySet["ImageClassificationAnnotation"]:
        """
        Return all manual image classification annotations associated with this frame.
        
        Returns:
            QuerySet: A queryset of related ImageClassificationAnnotation objects whose information source type is "manual_annotation".
        """
        return self.image_classification_annotations.filter(
            information_source__information_source_types__name="manual_annotation"
        )

    @property
    def has_predictions(self) -> bool:
        """
        Returns True if the frame has any associated prediction annotations.
        
        A prediction annotation is defined as an ImageClassificationAnnotation whose information source type is "prediction".
        """
        return self.predictions.exists()
    
    @property
    def has_manual_annotations(self) -> bool:
        """
        Returns True if the frame has any manual image classification annotations.
        
        Manual annotations are identified as related ImageClassificationAnnotation objects whose information source type is named "manual_annotation".
        """
        return self.manual_annotations.exists()

    def get_image(self) -> Optional[np.ndarray]:
        """
        Load and return the frame image as a NumPy array using OpenCV.
        
        Returns:
            The image as a NumPy array if successfully loaded, or None if the file does not exist or cannot be read.
        """
        frame_path = self.file_path
        if not frame_path.exists():
            logger.warning("Frame file not found at %s for Frame %s (Video %s)", frame_path, self.pk, self.video.uuid)
            return None
        try:
            image = cv2.imread(str(frame_path))
            if image is None:
                logger.warning("cv2.imread returned None for frame file %s (Frame %s, Video %s)", frame_path, self.pk, self.video.uuid)
            return image
        except Exception as e:
            logger.error("Error reading frame file %s (Frame %s, Video %s): %s", frame_path, self.pk, self.video.uuid, e, exc_info=True)
            return None

    def __str__(self):
        return f"Frame {self.frame_number} of Video {self.video.uuid}"

    def get_classification_annotations(self) -> models.QuerySet["ImageClassificationAnnotation"]:
        return self.image_classification_annotations.all()



from django.db import models
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ...medical import Endoscope, EndoscopyProcessor
    from ...administration import (
        CenterProduct,
        CenterWaste, CenterResource
    )
    from ...media import AnonymExaminationReport, AnonymHistologyReport

class CenterManager(models.Manager):
    def get_by_natural_key(self, name) -> "Center":
        return self.get(name=name)


class Center(models.Model):
    objects = CenterManager()

    # import_id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=255)
    display_name = models.CharField(max_length=255, blank=True, default="")

    first_names = models.ManyToManyField(
        to="FirstName",
        related_name="centers",
    )
    last_names = models.ManyToManyField("LastName", related_name="centers")

    if TYPE_CHECKING:
    #     first_names: RelatedManager["FirstName"]
    #     last_names: RelatedManager["LastName"]
        center_products: models.QuerySet["CenterProduct"]
        center_resources: models.QuerySet["CenterResource"]
        center_wastes: models.QuerySet["CenterWaste"]
        endoscopy_processors: models.QuerySet["EndoscopyProcessor"]
        endoscopes: models.QuerySet["Endoscope"]
        anonymexaminationreport_set: models.QuerySet["AnonymExaminationReport"]
        anonymhistologyreport_set: models.QuerySet["AnonymHistologyReport"]
        

    @classmethod
    def get_by_name(cls, name):
        return cls.objects.get(name=name)

    def natural_key(self) -> tuple[str]:
        return (self.name,)

    def save(self, *args, **kwargs):
        if not self.display_name:
            self.display_name = self.name
        super().save(*args, **kwargs)

    def __str__(self) -> str:
        return str(object=self.display_name or self.name)

    def get_first_names(self):
        return self.first_names.all()

    def get_last_names(self):
        return self.last_names.all()

    def get_endoscopes(self):
        """
        Returns all Endoscope instances associated with this center.
        """
        return self.endoscopes.all()



from django.db import models#
from typing import TYPE_CHECKING

class EndoscopeManager(models.Manager):
    def get_by_natural_key(self, name, sn):
        return self.get(name=name, sn=sn)

class Endoscope(models.Model):
    objects = EndoscopeManager()

    name = models.CharField(max_length=255) 
    sn = models.CharField(max_length=255)
    center = models.ForeignKey(
        'Center',
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name='endoscopes'
    )
    endoscope_type = models.ForeignKey(
        'EndoscopeType', 
        blank=True,
        null=True,
        on_delete=models.CASCADE,
        related_name='endoscopes'
    )

    if TYPE_CHECKING:
        from endoreg_db.models.center.center import Center
        center: "Center"
        endoscope_type: "EndoscopeType"

    def natural_key(self):
        return (self.name, self.sn)

    def __str__(self):
        return str(self.name)

    class Meta:
        ordering = ['name']
        verbose_name = 'Endoscope'
        verbose_name_plural = 'Endoscopes'

class EndoscopeTypeManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)

class EndoscopeType(models.Model):
    objects = EndoscopeTypeManager()

    name = models.CharField(max_length=255, unique=True)

    if TYPE_CHECKING:
        endoscopes: models.QuerySet["Endoscope"]

    def natural_key(self) -> tuple[str]:
        return (self.name,)
    
    def __str__(self):
        return str(self.name)

    class Meta:
        ordering = ['name']
        verbose_name = 'Endoscope Type'
        verbose_name_plural = 'Endoscope Types'



from .label import Label
from .label_set import LabelSet
from .label_type import LabelType
from .video_segmentation_label import VideoSegmentationLabel
from .video_segmentation_labelset import VideoSegmentationLabelSet
from .label_video_segment import (
    LabelVideoSegment,
)

from .annotation import (
    ImageClassificationAnnotation,
    VideoSegmentationAnnotation,
)

__all__ = [
    "Label",
    "LabelSet",
    "LabelType",
    "VideoSegmentationLabel",
    "VideoSegmentationLabelSet",
    "LabelVideoSegment",
    "ImageClassificationAnnotation",
    "VideoSegmentationAnnotation",
]


from django.shortcuts import render, get_object_or_404
from ...models import VideoFile
import json
from ...serializers.video import VideoFileSerializer

def video_timeline_view(request, video_id):
    """
    Fetches 'outside' label segments and passes their timestamps to the template.
    """
    video_entry = get_object_or_404(VideoFile, id=video_id)

    # Get metadata (Assumes serializer method exists)
    video_serializer = VideoFileSerializer(video_entry, context={'request': request})
    label_segments = video_serializer.get_label_time_segments(video_entry)

    segments = []
    frame_markers = []  # Store frame timestamps
    fps = 50  # Fixed FPS

    # Ensure "outside" label exists
    if "outside" in label_segments:
        for segment in label_segments["outside"]["time_ranges"]:
            segments.append({
                "start_time": segment["start_time"],
                "end_time": segment["end_time"]
            })

            # Extract frame timestamps for markers
            for frame_num, _ in segment["frames"].items():
                time_in_seconds = frame_num / fps
                frame_markers.append(time_in_seconds)

    # Set video duration correctly
    video_duration = video_entry.duration if hasattr(video_entry, "duration") and video_entry.duration else 226  # Default to 226 seconds

    print("Video ID:", video_id)
    print("Segments:", segments)
    print("Video Duration:", video_duration)

    return render(request, "timeline.html", {
        "video_id": video_id,
        "video_url": video_serializer.get_video_url(video_entry),
        "segments": json.dumps(segments),  # Send segment start/end times
        "frame_markers": json.dumps(frame_markers),  # Send frame timestamps
        "video_duration": video_duration,
    })



from endoreg_db.models import PatientFindingClassification
from endoreg_db.serializers.misc.translatable_field_mix_in import TranslatableFieldMixin


from rest_framework import serializers


class PatientFindingClassificationSerializer(serializers.ModelSerializer, TranslatableFieldMixin):
    """Serializer für PatientFinding-Klassifikationen"""

    classification_name = serializers.SerializerMethodField()
    classification_choice_name = serializers.SerializerMethodField()
    class Meta:
        model = PatientFindingClassification
        fields = ['id', 'classification', 'classification_name', 'classification_choice_name']

    def get_classification_name(self, obj:PatientFindingClassification):
        """
        Return the localized name for the classification attribute of a PatientFindingClassification instance.
        
        Parameters:
            obj (PatientFindingClassification): The instance whose classification name is to be localized.
        
        Returns:
            str: The localized classification name.
        """
        return self.get_localized_name(obj.classification)

    def get_classification_choice_name(self, obj:PatientFindingClassification):
        """
        Return the localized name for the classification choice of a patient finding classification instance.
        
        Parameters:
            obj (PatientFindingClassification): The patient finding classification instance.
        
        Returns:
            str: Localized name of the classification choice.
        """
        return self.get_localized_name(obj.classification_choice)


from .file_overview import FileOverviewSerializer
from .sensitive_patient_data import VoPPatientDataSerializer
from .stats import StatsSerializer
from .upload_job import UploadJobStatusSerializer, UploadCreateResponseSerializer
from .translatable_field_mix_in import TranslatableFieldMixin

__all__ = [
    "FileOverviewSerializer",
    "VoPPatientDataSerializer",
    "StatsSerializer",
    "UploadJobStatusSerializer",
    "UploadCreateResponseSerializer",
    "TranslatableFieldMixin"
]


from .correction import (
    VideoMetadataView,
    VideoProcessingHistoryView,
    VideoApplyMaskView,
    VideoRemoveFramesView,
    VideoCorrectionView,
)

from ..media.video_media import VideoMediaView

from .reimport import (
    VideoReimportView
)

from .segmentation import (
    VideoViewSet,
    VideoLabelView,
    UpdateLabelSegmentsView,
    rerun_segmentation,
)

# Phase 3.2: VideoStreamView moved to dedicated module
from .video_stream import VideoStreamView

from .timeline import (
    video_timeline_view
)

from .video_examination_viewset import VideoExaminationViewSet

__all__ = [
    # Video Correction (Phase 1.1) - Implemented
    'VideoMetadataView',
    'VideoProcessingHistoryView',
    'VideoApplyMaskView',
    'VideoRemoveFramesView',
    
    'VideoMediaView',
    
    'VideoCorrectionView',
    # 'TaskStatusView',
    # 'VideoDownloadProcessedView',

    # Reimport views
    'VideoReimportView',

    # Segmentation views
    'VideoViewSet',
    'VideoStreamView',
    'VideoLabelView',
    'UpdateLabelSegmentsView',
    'rerun_segmentation',

    # Timeline views
    'video_timeline_view',

    # Video Examination ViewSet
    "VideoExaminationViewSet",
]



from django.db import models
from .abstract import AbstractState
from typing import TYPE_CHECKING

class SensitiveMetaState(AbstractState):
    """State for sensitive meta data."""

    dob_verified = models.BooleanField(default=False)
    names_verified = models.BooleanField(default=False)

    origin = models.OneToOneField(
        "SensitiveMeta",
        on_delete=models.CASCADE,
        related_name="state",
        null=True,
        blank=True,
    )

    if TYPE_CHECKING:
        from endoreg_db.models import SensitiveMeta

        origin: "SensitiveMeta"

    @property
    def is_verified(self) -> bool:
        """
        Return True if both date of birth and names have been verified; otherwise, return False.
        """
        return self.dob_verified and self.names_verified

    def mark_dob_verified(self):
        """
        Set the date of birth verification status to True and persist the change to the database.
        """
        self.dob_verified = True
        self.save(update_fields=["dob_verified"])

    def mark_names_verified(self):
        """
        Mark the names as verified and persist the change to the database.
        """
        self.names_verified = True
        self.save(update_fields=["names_verified"])
    class Meta:
        verbose_name = "Sensitive Meta State"
        verbose_name_plural = "Sensitive Meta States"
